#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class BaseService:
    def __init__(self, connection=None, **kwargs):
        self._connection = connection
        self._connection_type = kwargs.get("connection_type")

    @property
    def connection_type(self):
        return self._connection_type

    @property
    def connection(self):
        if self._connection is None:
            # Create connection with defaults
            self.set_connection()
        return self._connection

    def set_connection(self, **kwargs):
        raise NotImplementedError



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class DeploymentTypes:
    KUBERNETES = "kubernetes"
    MINIKUBE = "minikube"
    MICRO_K8S = "microk8s"
    DOCKER_COMPOSE = "docker-compose"
    DOCKER = "docker"
    HEROKU = "heroku"

    VALUES = [KUBERNETES, MINIKUBE, MICRO_K8S, DOCKER_COMPOSE, DOCKER, HEROKU]


class DeploymentCharts:
    PLATFORM = "platform"
    AGENT = "agent"

    VALUES = [PLATFORM, AGENT]



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.proxies.schemas.base import get_config

OPTIONS = """
location = /robots.txt {{
    rewrite ^ /static/robots.txt;
}}
"""


def get_robots_config():
    return get_config(options=OPTIONS, indent=0)



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import click

from polyaxon.utils.formatting import Printer


@click.command()
@click.option(
    "--host",
    help="The service host.",
)
@click.option(
    "--port",
    type=int,
    help="The service port.",
)
@click.option(
    "--workers",
    type=int,
    help="Number of workers.",
)
@click.option(
    "--per-core",
    is_flag=True,
    default=False,
    help="To enable workers per core.",
)
@click.option(
    "--path",
    help="The service host.",
)
@click.option(
    "--uds",
    help="UNIX domain socket binding.",
)
def sandbox(host: str, port: int, workers: int, per_core: bool, path: str, uds: str):
    """Start a new sandbox session.

    This command is available starting from v1.18.
    """
    try:
        from polyaxon_deploy.cli.sandbox import start_sandbox

        return start_sandbox(
            host=host, port=port, workers=workers, per_core=per_core, path=path, uds=uds
        )

    except ImportError:
        Printer.print_error(
            "Please run 'pip install polyaxon[sandbox]' to start a sandbox session",
            sys_exit=True,
        )



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime
from typing import Any, Iterable, Optional

from kubernetes.client.rest import ApiException

from polyaxon.client import RunClient
from polyaxon.exceptions import PolyaxonK8SError
from polyaxon.k8s.manager import K8SManager
from polyaxon.utils.tz_utils import now
from traceml.logging import V1Log


def query_logs(
    k8s_manager: K8SManager,
    pod_id: str,
    container_id: str,
    stream: bool = False,
    since_seconds: int = None,
) -> Any:
    params = {}
    if stream:
        params = {"follow": True, "_preload_content": False}
    if since_seconds:
        params = {"since_seconds": since_seconds}

    return k8s_manager.k8s_api.read_namespaced_pod_log(
        pod_id, k8s_manager.namespace, container=container_id, timestamps=True, **params
    )


def process_log_line(log_line: str):

    if not isinstance(log_line, str):
        log_line = log_line.decode("utf-8")

    return V1Log.process_log_line(
        value=log_line.strip(), node=None, pod=None, container=None
    )


def stream_logs(
    k8s_manager: K8SManager, pod_id: str, container_id: str
) -> Iterable[str]:
    raw = None
    retries = 0
    no_logs = True

    while retries < 3 and no_logs:
        try:
            raw = query_logs(
                k8s_manager=k8s_manager,
                pod_id=pod_id,
                container_id=container_id,
                stream=True,
            )
        except (PolyaxonK8SError, ApiException):
            retries += 1

    if not raw:
        yield ""
    else:
        for log_line in raw.stream():
            if log_line:
                yield process_log_line(log_line=log_line)


def process_logs(
    k8s_manager: K8SManager,
    pod_id: str,
    container_id: str,
    filepath: str,
    since_seconds: int,
) -> bool:
    logs = None
    retries = 0
    no_logs = True
    while retries < 3 and no_logs:
        try:
            logs = query_logs(
                k8s_manager=k8s_manager,
                pod_id=pod_id,
                container_id=container_id,
                since_seconds=since_seconds,
            )
            no_logs = False
        except (PolyaxonK8SError, ApiException):
            retries += 1

    if not logs:
        return False

    log_lines = []
    for log_line in logs.split("\n"):
        if log_line:
            log_lines.append(process_log_line(log_line=log_line))

    # Creating the new file
    if not log_lines:
        return False

    with open(filepath, "w+") as destination:
        destination.write("\n".join(log_lines))

    return True


def sync_logs(
    k8s_manager: K8SManager,
    client: RunClient,
    last_check: Optional[datetime],
    pod_id: str,
    container_id: str,
    owner: str,
    project: str,
    run_uuid: str,
):
    new_check = now()
    since_seconds = None
    if last_check:
        since_seconds = (new_check - last_check).total_seconds()

        if since_seconds < 1:
            return last_check

    filepath = str(new_check.timestamp())
    created = process_logs(
        k8s_manager=k8s_manager,
        pod_id=pod_id,
        container_id=container_id,
        since_seconds=since_seconds,
        filepath=filepath,
    )
    if created:
        client.client.upload_run_logs(
            owner, project, run_uuid, uploadfile=filepath, path=filepath
        )
        return new_check

    return last_check



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

from collections.abc import Mapping
from typing import Dict


def sanitize_value(d, handle_dict: bool = False):
    if isinstance(d, str):
        return d
    if not isinstance(d, Mapping):
        return json.dumps(d)
    if not handle_dict:
        return json.dumps(d)
    return {d_k: sanitize_value(d_v, handle_dict=True) for d_k, d_v in d.items()}


def sanitize_string_dict(d: Dict[str, str] = None):
    if isinstance(d, Mapping):
        return {d_k: sanitize_value(d_v, handle_dict=False) for d_k, d_v in d.items()}
    return d



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import subprocess

from typing import List

from polyaxon.client import RunClient
from polyaxon.init.artifacts import download_artifact
from polyaxon.utils.formatting import Printer


def _download(
    connection_name: str,
    connection_kind: str,
    context_from: str,
    context_to: str,
    path_from: str,
    path_to: str,
):
    download_artifact(
        connection_name=connection_name,
        connection_kind=connection_kind,
        path_to=os.path.join(context_to, path_to),
        path_from=os.path.join(context_from, path_from, "outputs/tensorboard"),
        is_file=False,
        raise_errors=False,
        sync_fw=False,
        check_path=False,
    )


def _command(
    script_path: str,
    port: int,
    logdir: str,
    path_prefix: str,
    plugins: List[str] = None,
):
    cmd = []

    if plugins:
        cmd.append("pip install {}".format(" ".join(plugins)))

    cmd.append(
        f"tensorboard  --logdir={logdir} --port={port} --path_prefix={path_prefix} --host=0.0.0.0"
    )

    with open(script_path, "w") as script:
        script.write(" && ".join(cmd))
        subprocess.check_call(["chmod", "+x", script_path])
    Printer.print_success(
        "Tensorboard script is initialized, path: `{}`".format(script_path)
    )


def initialize_tensorboard(
    port: int,
    connection_name: str,
    connection_kind: str,
    context_from: str,
    context_to: str,
    uuids: List[str],
    use_names: bool,
    path_prefix: str,
    plugins: List[str],
):
    tensorboard_context = os.path.join(context_to, "tensorboard")
    if use_names:
        used_names = set([])
        for op in RunClient().list(query="uuid:{}".format("|".join(uuids))).results:
            op_name = op.name
            if op_name in used_names:
                op_name = f"{op.name}-{op.uuid[:10]}"
            else:
                used_names.add(op.name)
            _download(
                connection_name=connection_name,
                connection_kind=connection_kind,
                context_to=tensorboard_context,
                context_from=context_from,
                path_from=op.uuid,
                path_to=op_name,
            )
    else:
        for op in uuids:
            _download(
                connection_name=connection_name,
                connection_kind=connection_kind,
                context_to=tensorboard_context,
                context_from=context_from,
                path_from=op,
                path_to=op,
            )

    script_path = os.path.join(context_to, "start_tensorboard")
    _command(
        script_path=script_path,
        port=port,
        logdir=tensorboard_context,
        path_prefix=path_prefix,
        plugins=plugins,
    )



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow import fields, validate

import polyaxon_sdk

from polyaxon.polyflow.schedules.kinds import V1ScheduleKind
from polyaxon.schemas.base import BaseCamelSchema, BaseConfig


class CronScheduleSchema(BaseCamelSchema):
    kind = fields.Str(allow_none=True, validate=validate.Equal(V1ScheduleKind.CRON))
    start_at = fields.DateTime(allow_none=True)
    end_at = fields.DateTime(allow_none=True)
    cron = fields.String(required=True)
    max_runs = fields.Int(allow_none=True, validate=validate.Range(min=1))
    depends_on_past = fields.Bool(allow_none=True)

    @staticmethod
    def schema_config():
        return V1CronSchedule


class V1CronSchedule(BaseConfig, polyaxon_sdk.V1CronSchedule):
    """Cron schedules is an interface to trigger components repeatedly using a cron definition.

    Args:
        kind: str, should be equal to `cron`
        start_at: datetime, optional
        end_at: datetime, optional
        max_runs: int, optional
        cron: str, required
        depends_on_past: bool, optional


    ## YAML usage

    ```yaml
    >>> schedule:
    >>>   kind:
    >>>   startAt:
    >>>   endAt:
    >>>   maxRuns:
    >>>   cron:
    >>>   dependsOnPast:
    ```

    ## Python usage

    ```python
    >>> from datetime import datetime
    >>> from polyaxon.polyflow import V1CronSchedule
    >>> schedule = V1CronSchedule(
    >>>   start_at=datetime(...),
    >>>   end_at=datetime(...),
    >>>   max_runs=20,
    >>>   cron="0 0 * * *",
    >>>   dependsOnPast=False,
    >>> )
    ```

    ## Fields

    ### kind

    The kind signals to the CLI, client, and other tools that this schedule is a cron schedule.

    If you are using the python client to create the schedule,
    this field is not required and is set by default.

    ```yaml
    >>> run:
    >>>   kind: cron
    ```

    ### startAt

    Optional field to set the start time for kicking the first execution,
    all following executions will be relative to this time.

    ```yaml
    >>> run:
    >>>   startAt: "2019-06-24T21:20:07+00:00"
    ```

    ### endAt

    Optional field to set the end time for stopping this schedule.

    ```yaml
    >>> run:
    >>>   endAt: "2019-06-24T21:20:07+00:00"
    ```

    ### maxRuns

    The maximum number of times to execute the component.
    If used with end date, the schedule will terminate if one of the conditions is met.

    ```yaml
    >>> run:
    >>>   maxRuns: 10
    ```

    ### cron

    The cron expression to generate the component executions.

    ```yaml
    >>> run:
    >>>   cron: "0 0 * * *"
    ```

    ### dependsOnPast

    Optional field to tell Polyaxon to check if the
    previous execution was done before scheduling a new one, by default this is set to `false`.

    ```yaml
    >>> run:
    >>>   dependsOnPast: true
    ```
    """

    SCHEMA = CronScheduleSchema
    IDENTIFIER = V1ScheduleKind.CRON
    REDUCED_ATTRIBUTES = ["startAt", "endAt", "maxRuns", "dependsOnPast"]



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict

from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.exceptions import PolyaxonNotificationException
from polyaxon.notifiers.keys import INTEGRATIONS_WEBHOOKS_DISCORD
from polyaxon.notifiers.spec import NotificationSpec
from polyaxon.notifiers.webhook import WebHookNotifier


class DiscordWebHookNotifier(WebHookNotifier):
    notification_key = V1ConnectionKind.DISCORD
    name = "Discord WebHook"
    description = "Discord webhooks to send payload to a discord room."
    raise_empty_context = True
    config_key = INTEGRATIONS_WEBHOOKS_DISCORD

    @classmethod
    def serialize_notification_to_context(cls, notification: NotificationSpec) -> Dict:
        logo_url = ""

        payload = {"content": notification.get_details(), "avatar_url": logo_url}

        return payload

    @classmethod
    def _prepare(cls, context: Dict) -> Dict:
        context = super()._prepare(context)

        payload = {
            "username": "Polyaxon",
            "avatar_url": context.get("avatar_url"),
            "tts": context.get("tts", False),
        }
        content = context.get("content")
        if content and len(content) <= 2000:
            payload["content"] = content
        else:
            raise PolyaxonNotificationException(
                "Discord content must non null and 2000 or fewer characters."
            )

        proxy = context.get("proxy")
        if proxy:
            payload["https"] = proxy
        return payload



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

OPTIONS_OWNER = {
    "args": ["--owner", "-o"],
    "kwargs": dict(
        type=str,
        help="Name of the owner/namespace, "
        "if not provided it will default to the namespace of the current user.",
    ),
}

OPTIONS_NAME = {
    "args": ["--name", "-n"],
}

OPTIONS_PROJECT = {
    "args": ["--project", "-p"],
    "kwargs": dict(type=str, help="The project name, e.g. 'mnist' or 'acme/mnist'."),
}

OPTIONS_MODEL_VERSION = {
    "args": ["--version", "-ver"],
    "kwargs": dict(
        type=str,
        help="The component version, e.g. 'v1.3.4' or 'rc1' or 'latest'.",
    ),
}

OPTIONS_COMPONENT_VERSION = {
    "args": ["--version", "-ver"],
    "kwargs": dict(
        type=str,
        help="The component version, e.g. 'v1.3.4' or 'rc1' or 'latest'.",
    ),
}


OPTIONS_ARTIFACT_VERSION = {
    "args": ["--version", "-ver"],
    "kwargs": dict(
        type=str,
        help="The artifact version, e.g. 'v1.3.4' or 'rc1' or 'latest'.",
    ),
}

OPTIONS_RUN_UID = {
    "args": ["--uid", "-uid"],
    "kwargs": dict(type=str, help="The run uuid."),
}



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import click

from polyaxon.logger import clean_outputs
from polyaxon.utils.formatting import Printer


@click.command()
@click.argument("shell")
@clean_outputs
def completion(shell):
    """Show documentation for installing the auto-completion for polyaxon cli.

    Valid options: [bash, zsh, fish]

    \b
    $ polyaxon completion SHELL
    """

    if shell == "bash":
        Printer.print_heading(
            "Please save the following scripts:",
        )
        Printer.print(
            "_POLYAXON_COMPLETE=bash_source polyaxon > ~/.polyaxon-complete.bash"
        )
        Printer.print("_PLX_COMPLETE=bash_source plx > ~/.plx-complete.bash")
        Printer.print_heading(
            "Add the following lines to your: `~/.bashrc`",
        )
        Printer.print_text(
            "# Polyaxon completion\n. ~/.polyaxon-complete.bash\n. ~/.plx-complete.bash"
        )
        Printer.print_heading(
            "Reload your shell.",
        )
    elif shell == "zsh":
        Printer.print_heading(
            "Please save the following scripts:",
        )
        Printer.print(
            "_POLYAXON_COMPLETE=zsh_source polyaxon > ~/.polyaxon-complete.zsh"
        )
        Printer.print("_PLX_COMPLETE=zsh_source plx > ~/.plx-complete.zsh")
        Printer.print_heading(
            "Add the following lines to your: `~/.zshrc`",
        )
        Printer.print_text(
            "# Polyaxon completion\n. ~/.polyaxon-complete.zsh\n. ~/.plx-complete.zsh"
        )
        Printer.print_heading(
            "Reload your shell.",
        )
    elif shell == "fish":
        Printer.print_heading(
            "Please save the following scripts under `~/.config/fish/completions`:",
        )
        Printer.print(
            "_POLYAXON_COMPLETE=fish_source polyaxon > ~/.config/fish/completions/polyaxon-complete.fish"  # noqa
        )
        Printer.print(
            "_PLX_COMPLETE=fish_source plx > ~/.config/fish/completions/plx-complete.fish"
        )
    else:
        Printer.print("Shell {} is not supported.".format(shell))



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, List, Optional

from polyaxon.polyflow import V1Init
from polyaxon.schemas.types import V1ConnectionType
from polyaxon.utils.list_utils import to_list


def get_connection_annotations(
    artifacts_store: Optional[V1ConnectionType],
    init_connections: Optional[List[V1Init]],
    connections: List[str],
    connection_by_names: Optional[Dict[str, V1ConnectionType]],
) -> Dict:
    """Resolve all annotations to inject per replica"""
    connections = to_list(connections, check_none=True)
    init_connections = to_list(init_connections, check_none=True)
    connection_by_names = connection_by_names or {}

    requested_connection_names = connections[:]
    for init_connection in init_connections:
        if (
            init_connection.connection
            and init_connection.connection not in requested_connection_names
        ):
            requested_connection_names.append(init_connection.connection)
    if artifacts_store and artifacts_store.name not in requested_connection_names:
        requested_connection_names.append(artifacts_store.name)

    annotations = {}
    for c in requested_connection_names:
        annotations.update(connection_by_names[c].annotations or {})

    return annotations



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging

from collections.abc import Mapping

try:
    import numpy as np
except ImportError:
    np = None


def to_list(value, check_none=False, check_dict=False, to_unique: bool = False):
    def _to_unique(v):
        try:
            return list(dict.fromkeys(v))
        except Exception as e:
            logging.debug("Could not return unique value for list. Error %s", e)
            return list(v)

    if check_none and value is None:
        return []
    if isinstance(value, (list, tuple, set)):
        return _to_unique(value) if to_unique else list(value)
    if np and isinstance(value, np.ndarray):
        value = value.tolist()
        return _to_unique(value) if to_unique else value
    if check_dict and isinstance(value, Mapping):
        return list(value.items())
    return [value]



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import Dict


def get_query_params(
    limit: str = None, offset: str = None, query: str = None, sort: str = None
) -> Dict:
    params = {}
    if limit:
        params["limit"] = limit
    if offset:
        params["offset"] = offset
    if query:
        params["query"] = query
    if sort:
        params["sort"] = sort

    return params


def get_logs_params(last_time: str = None, last_file: str = None) -> Dict:
    params = {}
    if last_file:
        params["last_file"] = last_file
    if last_time:
        params["last_time"] = last_time

    return params



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow import fields, validate

import polyaxon_sdk

from polyaxon.polyflow.early_stopping import EarlyStoppingSchema
from polyaxon.polyflow.matrix.base import BaseSearchConfig
from polyaxon.polyflow.matrix.kinds import V1MatrixKind
from polyaxon.polyflow.matrix.params import HpParamSchema
from polyaxon.schemas.base import BaseCamelSchema
from polyaxon.schemas.fields.ref_or_obj import RefOrObject


class RandomSearchSchema(BaseCamelSchema):
    kind = fields.Str(allow_none=True, validate=validate.Equal(V1MatrixKind.RANDOM))
    params = fields.Dict(
        keys=fields.Str(), values=fields.Nested(HpParamSchema), required=True
    )
    num_runs = RefOrObject(
        fields.Int(required=True, validate=validate.Range(min=1)), required=True
    )
    seed = RefOrObject(fields.Int(allow_none=True))
    concurrency = RefOrObject(fields.Int(allow_none=True))
    early_stopping = fields.List(fields.Nested(EarlyStoppingSchema), allow_none=True)

    @staticmethod
    def schema_config():
        return V1RandomSearch


class V1RandomSearch(BaseSearchConfig, polyaxon_sdk.V1RandomSearch):
    """Random search creates a number of unique experiments by sampling randomly
    from a search space.
    Random search is a competitive method for black-box parameter tuning in machine learning.

    Random search requires a parameter `numRuns`,
    this is essential because Polyaxon needs to know how many experiments to sample.

    Args:
        kind: str, should be equal `grid`
        params: List[Dict[str, [params](/docs/automation/optimization-engine/params/)]]
        concurrency: int, optional
        num_runs: int, optional
        seed: int, optional
        early_stopping: List[[EarlyStopping](/docs/automation/helpers/early-stopping)], optional

    ## YAML usage

    ```yaml
    >>> matrix:
    >>>   kind: random
    >>>   concurrency:
    >>>   params:
    >>>   numRuns:
    >>>   seed:
    >>>   earlyStopping:
    ```

    ## Python usage

    ```python
    >>> from polyaxon.polyflow import (
    >>>     V1RandomSearch, V1HpLogSpace, V1HpUniform, V1FailureEarlyStopping, V1MetricEarlyStopping
    >>> )
    >>> matrix = V1RandomSearch(
    >>>   num_runs=20,
    >>>   concurrency=2,
    >>>   seed=23,
    >>>   params={"param1": V1HpLogSpace(...), "param2": V1HpUniform(...), ... },
    >>>   early_stopping=[V1FailureEarlyStopping(...), V1MetricEarlyStopping(...)]
    >>> )
    ```

    ## Fields

    ### kind

    The kind signals to the CLI, client, and other tools that this matrix is a random search.

    If you are using the python client to create the mapping,
    this field is not required and is set by default.

    ```yaml
    >>> matrix:
    >>>   kind: random
    ```

    ### concurrency

    An optional value to set the number of concurrent operations.

    <blockquote class="light">
    This value only makes sense if less or equal to the total number of possible runs.
    </blockquote>

    ```yaml
    >>> matrix:
    >>>   kind: random
    >>>   concurrency: 2
    ```

    For more details about concurrency management,
    please check the [concurrency section](/docs/automation/helpers/concurrency/).

    ### params

    A dictionary of `key -> value generator`
    to generate the parameters.

    To learn about all possible
    [params generators](/docs/automation/optimization-engine/params/).

    > The parameters generated will be validated against
    > the component's inputs/outputs definition to check that the values
    > can be passed and have valid types.

    ```yaml
    >>> matrix:
    >>>   kind: random
    >>>   params:
    >>>     param1:
    >>>        kind: ...
    >>>        value: ...
    >>>     param2:
    >>>        kind: ...
    >>>        value: ...
    ```

    ### numRuns

    Maximum number of runs to start based on the search space defined.

    ```yaml
    >>> matrix:
    >>>   kind: random
    >>>   numRuns: 5
    ```

    ### seed

    Since this algorithm uses random generators,
    if you want to control the seed for the random generator, you can pass a seed.

     ```yaml
    >>> matrix:
    >>>   kind: random
    >>>   seed: 523
    ```

    ### earlyStopping

    A list of early stopping conditions to check for terminating
    all operations managed by the pipeline.
    If one of the early stopping conditions is met,
    a signal will be sent to terminate all running and pending operations.

    ```yaml
    >>> matrix:
    >>>   kind: random
    >>>   earlyStopping: ...
    ```

    For more details please check the
    [early stopping section](/docs/automation/helpers/early-stopping/).

    ## Example

    In this example the random search algorithm will try 20 unique experiments based on the
    search space defined in the params subsection.


    ```yaml
    >>> version: 1.1
    >>> kind: operation
    >>> matrix:
    >>>   kind: random
    >>>   concurrency: 10
    >>>   numRuns: 20
    >>>   params:
    >>>     lr:
    >>>       kind: logspace
    >>>       value: 0.01:0.1:5
    >>>     dropout:
    >>>       kind: choice
    >>>       value: [0.2, 0.5]
    >>>    activation:
    >>>       kind: pchoice
    >>>       value: [[elu, 0.1], [relu, 0.2], [sigmoid, 0.7]]
    >>>    early_stopping:
    >>>      - metric: accuracy
    >>>        value: 0.9
    >>>        optimization: maximize
    >>>      - metric: loss
    >>>        value: 0.05
    >>>        optimization: minimize
    >>> component:
    >>>   inputs:
    >>>     - name: batch_size
    >>>       type: int
    >>>       isOptional: true
    >>>       value: 128
    >>>     - name: lr
    >>>       type: float
    >>>     - name: dropout
    >>>       type: float
    >>>     - name: activation
    >>>       type: str
    >>>   container:
    >>>     image: image:latest
    >>>     command: [python3, train.py]
    >>>     args: [
    >>>         "--batch-size={{ batch_size }}",
    >>>         "--lr={{ lr }}",
    >>>         "--dropout={{ dropout }}",
    >>>         "--activation={{ activation }}"
    >>>     ]
    ```
    """

    SCHEMA = RandomSearchSchema
    IDENTIFIER = V1MatrixKind.RANDOM
    REDUCED_ATTRIBUTES = ["numRuns", "seed", "concurrency", "earlyStopping"]



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict

from polyaxon.k8s import k8s_schemas


def get_custom_object(
    namespace: str,
    resource_name: str,
    kind: str,
    api_version: str,
    labels: Dict,
    annotations: Dict,
    custom_object: Dict,
) -> Dict:
    metadata = k8s_schemas.V1ObjectMeta(
        name=resource_name,
        labels=labels,
        annotations=annotations,
        namespace=namespace,
    )
    custom_object.update(
        {"kind": kind, "apiVersion": api_version, "metadata": metadata}
    )

    return custom_object



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import List, Union

from fsspec import AbstractFileSystem

from polyaxon.logger import logger
from polyaxon.utils.list_utils import to_list
from polyaxon.utils.path_utils import check_or_create_path


def download_file_or_dir(
    fs: AbstractFileSystem,
    path_from: str,
    path_to: str,
    is_file: bool,
    check_path: bool,
):
    if check_path:
        is_file = fs.isfile(path_from)
    check_or_create_path(path_to, is_dir=not is_file)
    fs.download(rpath=path_from, lpath=path_to, recursive=not is_file)


def delete_file_or_dir(
    fs: AbstractFileSystem,
    subpath: Union[str, List[str]],
    is_file: bool = False,
):
    subpath = to_list(subpath, check_none=True)
    for sp in subpath:
        try:
            fs.delete(path=sp, recursive=not is_file)
        except Exception as e:
            logger.info("Could not delete path %s\nError: %s", sp, e)



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.constants.globals import UNKNOWN
from traceml.processors.units_processors import to_cpu_value, to_memory_bytes


class NodeLifeCycle:
    UNKNOWN = UNKNOWN
    READY = "ready"
    NOT_READY = "notReady"
    DELETED = "deleted"

    CHOICES = (
        (UNKNOWN, UNKNOWN),
        (READY, READY),
        (NOT_READY, NOT_READY),
        (DELETED, DELETED),
    )


class NodeParser:
    @staticmethod
    def get_status(node) -> str:
        status = [c.status for c in node.status.conditions if c.type == "Ready"][0]
        if status == "True":
            return NodeLifeCycle.READY
        if status == "False":
            return NodeLifeCycle.NOT_READY
        return NodeLifeCycle.UNKNOWN

    @staticmethod
    def get_n_gpus(node) -> int:
        if "gpu" not in node.status.allocatable:
            return 0
        return int(node.status.allocatable.get("nvidia.com/gpu", 0))

    @staticmethod
    def get_cpu(node) -> float:
        cpu = node.status.allocatable["cpu"]
        return to_cpu_value(cpu)

    @classmethod
    def get_memory(cls, node) -> int:
        return to_memory_bytes(node.status.allocatable["memory"])

    @staticmethod
    def get_runtime(node) -> str:
        return node.status.node_info.container_runtime_version

    @staticmethod
    def get_schedulable_state(node) -> bool:
        return not node.spec.unschedulable

    @staticmethod
    def get_addresses(node) -> str:
        return node.status.addresses



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import click

from polyaxon import settings
from polyaxon.cli.admin import admin
from polyaxon.cli.artifacts import artifacts
from polyaxon.cli.auth import login, logout, whoami
from polyaxon.cli.check import check
from polyaxon.cli.completion import completion
from polyaxon.cli.components import components
from polyaxon.cli.config import config
from polyaxon.cli.dashboard import dashboard
from polyaxon.cli.init import init
from polyaxon.cli.models import models
from polyaxon.cli.operations import ops
from polyaxon.cli.port_forward import port_forward
from polyaxon.cli.projects import project
from polyaxon.cli.run import run
from polyaxon.cli.services.sandbox import sandbox
from polyaxon.cli.session import set_versions_config
from polyaxon.cli.version import check_cli_version, upgrade, version
from polyaxon.logger import clean_outputs, configure_logger
from polyaxon.services.values import PolyaxonServices
from polyaxon.utils.bool_utils import to_bool
from polyaxon.utils.formatting import Printer

DOCS_GEN = to_bool(os.environ.get("POLYAXON_DOCS_GEN", False))


@click.group()
@click.option(
    "-v", "--verbose", is_flag=True, default=False, help="Turn on debug logging"
)
@click.option(
    "--offline",
    is_flag=True,
    default=False,
    help="Run command in offline mode if supported. "
    "Currently used for run command in --local mode.",
)
@click.pass_context
@clean_outputs
def cli(context, verbose, offline):
    """Polyaxon - Cloud Native Machine Learning Automation & Experimentation tool.

    This CLI provides tools to:

      - Parse, Validate, and Check Polyaxonfiles.

      - Interact with Polyaxon server.

      - Run and Monitor experiments and jobs.

    This CLI tool comes with a caching mechanism:

      - You can initialize a project with: polyaxon init -p [project name]

      - Otherwise Polyaxon will use the default global path for the cache.

    You can check the version of your CLI by running:

      - polyaxon version

    You can check the version of the CLI and the server, and the compatibility matrix with:

      - polyaxon version --check

    To enable the debug mode, you can use the `-v` flag:

      - polyaxon -v ...

    To configure your host:

      - polyaxon config set --host=...

    To check your current config:

      - polyaxon config show

    Common commands:

      - polyaxon project get

      - polyaxon run [-f] [-l]

      - polyaxon ops ls

      - polyaxon ops logs

      - polyaxon ops get

      - polyaxon config set ...

    Admin deployment commands:

      - polyaxon admin deploy [-f] [--check]

      - polyaxon admin upgrade [-f] [--check]

      - polyaxon admin teardown [-f]

    For more information, please visit https://polyaxon.com/docs/core/cli/

    Check the help available for each command listed below by appending `-h`.
    """
    settings.set_cli_config()
    configure_logger(verbose)
    if settings.CLIENT_CONFIG.no_op:
        Printer.print_warning(
            "POLYAXON_NO_OP is set to `true`, some commands will not function correctly."
        )
    context.obj = context.obj or {}
    if not settings.CLIENT_CONFIG.client_header:
        settings.CLIENT_CONFIG.set_cli_header()
    context.obj["offline"] = offline
    if offline:
        os.environ["POLYAXON_IS_OFFLINE"] = "true"
        settings.CLIENT_CONFIG.is_offline = True
    non_check_cmds = [
        "completion",
        "config",
        "version",
        "login",
        "logout",
        "deploy",
        "admin",
        "teardown",
        "docker",
        "initializer",
        "sidecar",
        "proxy",
        "notify",
        "upgrade",
        "port-forward",
    ]
    if (
        not (
            context.invoked_subcommand in non_check_cmds
            or offline
            or settings.CLIENT_CONFIG.no_api
            or PolyaxonServices.SERVICE
            or DOCS_GEN
        )
        and not settings.CLI_CONFIG.installation
    ):
        cli_config = set_versions_config(is_cli=False)
        settings.CLI_CONFIG = cli_config
        check_cli_version(cli_config, is_cli=False)


cli.add_command(login)
cli.add_command(logout)
cli.add_command(whoami)
cli.add_command(upgrade)
cli.add_command(version)
cli.add_command(config)
cli.add_command(check)
cli.add_command(init)
cli.add_command(project)
cli.add_command(ops)
cli.add_command(artifacts)
cli.add_command(components)
cli.add_command(models)
cli.add_command(run)
cli.add_command(dashboard)
cli.add_command(admin)
cli.add_command(port_forward)
cli.add_command(completion)
cli.add_command(sandbox)

# INIT
if PolyaxonServices.is_init():
    from polyaxon.cli.services.clean_artifacts import clean_artifacts
    from polyaxon.cli.services.docker import docker
    from polyaxon.cli.services.initializer import initializer
    from polyaxon.cli.services.wait import wait

    cli.add_command(clean_artifacts)
    cli.add_command(docker)
    cli.add_command(initializer)
    cli.add_command(wait)

# Events
if PolyaxonServices.is_events_handlers():
    from polyaxon.cli.services.notifier import notify

    cli.add_command(notify)

# Sidecar
if PolyaxonServices.is_sidecar():
    from polyaxon.cli.services.sidecar import sidecar

    cli.add_command(sidecar)

# Tuner
if PolyaxonServices.is_hp_search():
    from polyaxon.cli.services.tuner import tuner

    cli.add_command(tuner)

# Agents
if PolyaxonServices.is_agent():
    from polyaxon.cli.services.agent import agent

    cli.add_command(agent)

# Proxies
if (
    PolyaxonServices.is_streams()
    or PolyaxonServices.is_api()
    or PolyaxonServices.is_gateway()
):
    from polyaxon.cli.services.proxies import proxy

    cli.add_command(proxy)

# Streams
if PolyaxonServices.is_streams():
    from polyaxon.cli.services.streams import streams

    cli.add_command(streams)


def main():
    cli(auto_envvar_prefix="POLYAXON_CLI")



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, List, Optional

from polyaxon.polyflow import V1IO, V1CompiledOperation
from polyaxon.schemas.types import V1ConnectionType
from polyaxon.types import IMAGE, LINEAGE_VALUES
from polyaxon.utils.list_utils import to_list
from traceml.artifacts import V1ArtifactKind, V1RunArtifact


def collect_artifacts_from_io(
    io: V1IO, connection_by_names: Dict[str, V1ConnectionType], is_input: bool
) -> Optional[V1RunArtifact]:
    if io.type not in LINEAGE_VALUES:
        return None

    if io.type == IMAGE:
        image = io.value
        connection = connection_by_names.get(io.connection)
        if connection and connection.schema and connection.schema.url:
            image = "{}/{}".format(connection.schema.url, image)
        return V1RunArtifact(
            name=io.name,
            kind=V1ArtifactKind.DOCKER_IMAGE,
            connection=io.connection,
            summary={"image": image},
            is_input=is_input,
        )


def collect_artifacts_from_io_section(
    io_section: List[V1IO],
    connection_by_names: Dict[str, V1ConnectionType],
    is_input: bool,
) -> List[V1RunArtifact]:
    io_section = to_list(io_section, check_none=True)
    artifacts = [
        collect_artifacts_from_io(
            io, connection_by_names=connection_by_names, is_input=is_input
        )
        for io in io_section
    ]
    return [a for a in artifacts if a]


def collect_io_artifacts(
    compiled_operation: V1CompiledOperation,
    connection_by_names: Dict[str, V1ConnectionType],
) -> List[V1RunArtifact]:
    connection_by_names = connection_by_names or {}
    artifacts = []
    artifacts += collect_artifacts_from_io_section(
        compiled_operation.inputs,
        connection_by_names=connection_by_names,
        is_input=True,
    )
    artifacts += collect_artifacts_from_io_section(
        compiled_operation.outputs,
        connection_by_names=connection_by_names,
        is_input=False,
    )
    return artifacts



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os

from typing import Dict, List, Optional, Union

import aiofiles

from polyaxon import settings
from polyaxon.fs.tar import tar_dir
from polyaxon.fs.tar import tar_files as sync_tar_files
from polyaxon.fs.types import FSSystem
from polyaxon.lifecycle import V1ProjectFeature
from polyaxon.logger import logger
from polyaxon.utils.coroutine import run_sync
from polyaxon.utils.hashing import hash_value
from polyaxon.utils.list_utils import to_list
from polyaxon.utils.path_utils import check_or_create_path


async def ensure_async_execution(
    fs: FSSystem, fct: str, is_async: bool = False, *args, **kwargs
):
    async_fct = "_{}".format(fct)
    if is_async and hasattr(fs, async_fct):
        return await getattr(fs, async_fct)(*args, **kwargs)
    return await run_sync(getattr(fs, fct), *args, **kwargs)


async def upload_data(fs: FSSystem, subpath: str, data):
    path_to = settings.AGENT_CONFIG.get_store_path(
        subpath=subpath, entity=V1ProjectFeature.RUNTIME
    )
    try:
        return await ensure_async_execution(
            fs=fs,
            fct="pipe",
            is_async=fs.async_impl,
            path=path_to,
            value=data.encode(),
        )
    except Exception as e:
        logger.warning("Could not upload %s. Error %s" % (subpath, e))
        return None


async def upload_file(fs: FSSystem, subpath: str):
    path_from = settings.AGENT_CONFIG.get_local_path(
        subpath=subpath, entity=V1ProjectFeature.RUNTIME
    )
    path_to = settings.AGENT_CONFIG.get_store_path(
        subpath=subpath, entity=V1ProjectFeature.RUNTIME
    )
    try:
        return await ensure_async_execution(
            fs=fs,
            fct="put",
            is_async=fs.async_impl,
            lpath=path_from,
            rpath=path_to,
            recursive=False,
        )
    except Exception as e:
        logger.warning("Could not upload %s. Error %s" % (path_from, e))
        return None


async def check_is_file(fs: FSSystem, subpath: str) -> bool:
    filepath = settings.AGENT_CONFIG.get_store_path(
        subpath=subpath, entity=V1ProjectFeature.RUNTIME
    )
    try:
        return await ensure_async_execution(
            fs=fs,
            fct="isfile",
            is_async=fs.async_impl,
            path=filepath,
        )
    except Exception as e:
        logger.warning(
            "Could not upload check the details of the path %s. Error %s"
            % (filepath, e)
        )
        return False


async def upload_dir(fs: FSSystem, subpath: str) -> Optional[str]:
    path_from = settings.AGENT_CONFIG.get_local_path(
        subpath=subpath, entity=V1ProjectFeature.RUNTIME
    )
    path_to = settings.AGENT_CONFIG.get_store_path(
        subpath=subpath, entity=V1ProjectFeature.RUNTIME
    )
    try:
        return await ensure_async_execution(
            fs=fs,
            fct="put",
            is_async=fs.async_impl,
            lpath=path_from,
            rpath=path_to,
            recursive=True,
        )
    except Exception as e:
        logger.warning("Could not upload %s. Error %s" % (path_from, e))
        return None


async def download_file(
    fs: FSSystem, subpath: str, check_cache: bool = True
) -> Optional[str]:
    path_from = settings.AGENT_CONFIG.get_store_path(
        subpath=subpath, entity=V1ProjectFeature.RUNTIME
    )
    path_to = os.path.join(settings.CLIENT_CONFIG.archive_root, subpath)

    if os.path.exists(path_to):
        if check_cache:
            # file already exists
            return path_to
        else:
            os.remove(path_to)

    try:
        check_or_create_path(path_to, is_dir=False)
    except Exception as e:
        logger.warning("Error during async dir creation. Error %s %s" % (path_from, e))
        pass
    try:
        await ensure_async_execution(
            fs=fs,
            fct="get",
            is_async=fs.async_impl,
            rpath=path_from,
            lpath=path_to,
            recursive=False,
        )
        return path_to
    except Exception as e:
        logger.warning("Could not download %s. Error %s" % (path_from, e))
        return None


async def download_files(
    fs: FSSystem,
    subpaths: List[str],
    check_cache: bool = True,
    pkg_files: List[str] = None,
) -> List[str]:
    pkg_files = to_list(pkg_files, check_none=True)
    for subpath in subpaths:
        try:
            file_to_path = await download_file(
                fs=fs, subpath=subpath, check_cache=check_cache
            )
            pkg_files.append(file_to_path)
        except Exception as e:
            logger.warning(
                "The file download for path %s failed. " "Error %s" % (subpath, e)
            )

    return pkg_files


async def open_file(
    fs: FSSystem, subpath: str, check_cache: bool = True
) -> Optional[str]:
    path_from = settings.AGENT_CONFIG.get_store_path(
        subpath=subpath, entity=V1ProjectFeature.RUNTIME
    )
    path_to = os.path.join(settings.CLIENT_CONFIG.archive_root, subpath)

    if os.path.exists(path_to):
        if check_cache:
            # file already exists
            async with aiofiles.open(path_to, mode="r") as f:
                return await f.read()
        else:
            os.remove(path_to)

    try:
        check_or_create_path(path_to, is_dir=False)
    except Exception as e:
        logger.warning("Error during async dir creation. Error %s %s" % (path_from, e))
        pass
    try:
        content = await ensure_async_execution(
            fs=fs,
            fct="cat",
            is_async=fs.async_impl,
            path=path_from,
            recursive=False,
        )
        try:
            content = content.decode()
        except Exception as e:
            logger.warning("Could not decode content from bytes, error: %s", e)
        async with aiofiles.open(path_to, "w") as fw:
            await fw.write(content)
        return content
    except Exception as e:
        logger.warning("Could not download %s. Error %s" % (path_from, e))
        return None


async def download_dir(
    fs: FSSystem, subpath: str, to_tar: bool = False
) -> Optional[str]:
    path_from = settings.AGENT_CONFIG.get_store_path(
        subpath=subpath, entity=V1ProjectFeature.RUNTIME
    )
    path_to = os.path.join(settings.CLIENT_CONFIG.archive_root, subpath)
    check_or_create_path(path_to, is_dir=True)
    try:
        await ensure_async_execution(
            fs=fs,
            fct="get",
            is_async=fs.async_impl,
            rpath=path_from,
            lpath=path_to,
            recursive=True,
        )
        if not os.path.exists(path_to):
            return None
        if to_tar:
            return await run_sync(tar_dir, path_to)
        return path_to
    except Exception as e:
        logger.warning("Could not download %s. Error %s" % (path_from, e))
        return None


async def download_dirs(
    fs: FSSystem,
    subpaths: List[str],
    pkg_files: List[str] = None,
    to_tar: bool = False,
    tar_filename: str = None,
) -> Union[str, List[str]]:
    pkg_files = to_list(pkg_files, check_none=True)
    for subpath in subpaths:
        try:
            file_to_path = await download_dir(fs=fs, subpath=subpath, to_tar=False)
            pkg_files.append(file_to_path)
        except Exception as e:
            logger.warning(
                "The file download for path %s failed. " "Error %s" % (subpath, e)
            )

    if to_tar:
        tar_filename = tar_filename or "download"
        return await tar_files(
            filename="{}.pkg.{}".format(tar_filename, hash_value(subpaths)),
            pkg_files=pkg_files,
            subpath=tar_filename,
        )
    return pkg_files


async def list_files(
    fs: FSSystem, subpath: str, filepath: str = None, force: bool = False
) -> Dict:
    store_path = settings.AGENT_CONFIG.get_store_path(
        subpath=subpath, entity=V1ProjectFeature.RUNTIME
    )
    if filepath:
        store_path = os.path.join(store_path, filepath)
    try:
        paths = await ensure_async_execution(
            fs=fs,
            fct="ls",
            is_async=fs.async_impl,
            path=store_path,
            detail=True,
            force=force,
        )
        results = {"files": {}, "dirs": []}
        for p in paths:
            name = os.path.basename(p["name"])
            if p.get("type") == "file":
                results["files"][name] = p.get("size")
            else:
                results["dirs"].append(name)
        return results
    except Exception as e:
        error = "Could not list path %s. Error %s" % (subpath, e)
        logger.warning(error)
        return {"files": {}, "dirs": [], "error": error}


async def delete_file_or_dir(
    fs: FSSystem, subpath: Union[str, List[str]], is_file: bool
) -> bool:
    try:
        await ensure_async_execution(
            fs=fs,
            fct="rm",
            is_async=fs.async_impl,
            path=settings.AGENT_CONFIG.get_store_path(
                subpath=subpath, entity=V1ProjectFeature.RUNTIME
            ),
            recursive=not is_file,
        )
        return True
    except Exception as e:
        logger.warning("Could not delete %s. Error %s" % (subpath, e))
        return False


async def tar_files(filename: str, pkg_files: List[str], subpath: str = None) -> str:
    relative_to = (
        os.path.join(settings.CLIENT_CONFIG.archive_root, subpath) if subpath else None
    )
    return await run_sync(
        sync_tar_files,
        filename,
        pkg_files,
        relative_to,
    )



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict

from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.notifiers.keys import INTEGRATIONS_WEBHOOKS_MATTERMOST
from polyaxon.notifiers.spec import NotificationSpec
from polyaxon.notifiers.webhook import WebHookNotifier


class MattermostWebHookNotifier(WebHookNotifier):
    notification_key = V1ConnectionKind.MATTERMOST
    name = "Mattermost WebHook"
    description = "Mattermost webhooks to send payload to a Mattermost channel."
    raise_empty_context = True
    config_key = INTEGRATIONS_WEBHOOKS_MATTERMOST
    validate_keys = ["channel"]

    @classmethod
    def serialize_notification_to_context(cls, notification: NotificationSpec) -> Dict:
        payload = {
            "title": notification.get_title(),
            "text": notification.get_details(),
            "color": notification.get_color(),
            "fields": [],
            "author_name": "Polyaxon",
            "author_link": cls.get_url(notification),
            "author_icon": None,
        }

        return payload

    @classmethod
    def _prepare(cls, context):
        context = super()._prepare(context)

        data = {
            "pretext": context.get("pretext"),
            "title": context.get("title"),
            "text": context.get("text"),
            "color": context.get("color"),
            "fields": context.get("fields"),
            "author_name": context.get("author_name", "Polyaxon"),
            "author_link": context.get("author_link", "https://polyaxon.com"),
            "author_icon": context.get("author_icon"),
        }
        return {"attachments": [data]}

    @classmethod
    def _pre_execute_web_hook(cls, data, config):
        channel = config.get("channel")
        if channel:
            data["channel"] = channel

        return data



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import polyaxon_sdk

from polyaxon import settings
from polyaxon.constants.globals import NO_AUTH


class ApiClient(polyaxon_sdk.ApiClient):
    def call_api(
        self,
        resource_path,
        method,
        path_params=None,
        query_params=None,
        header_params=None,
        body=None,
        post_params=None,
        files=None,
        response_types_map=None,
        auth_settings=None,
        async_req=None,
        _return_http_data_only=None,
        collection_formats=None,
        _preload_content=True,
        _request_timeout=None,
        _host=None,
        _request_auth=None,
    ):
        if response_types_map and 200 in response_types_map:
            response_types_map[201] = response_types_map[200]
        return super().call_api(
            resource_path,
            method,
            path_params,
            query_params=query_params,
            header_params=header_params,
            body=body,
            post_params=post_params,
            files=files,
            response_types_map=response_types_map,
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            collection_formats=collection_formats,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            _host=_host,
            _request_auth=_request_auth,
        )


class PolyaxonClient:
    """Auto-configurable and high-level base client that abstract
    the need to set a configuration for each service.

    PolyaxonClient comes with logic
    to pass config and token to other specific clients.

    If no values are passed to this class,
    Polyaxon will try to resolve the configuration from the environment:
     * If you have a configured CLI, Polyaxon will use the configuration of the cli.
     * If you use this client in the context of a job or a service managed by Polyaxon,
       a configuration will be available.

    > N.B. PolyaxonClient requires python >= 3.5,
    if you want to interact with Polyaxon using a client
    compatible with python 2.7 please check polyaxon-sdk.

    Args:
        config: ClientConfig, optional, Instance of a ClientConfig.
        token: str, optional, the token to use for authenticating the clients,
               if the user is already logged in using the CLI, it will automatically use that token.
               Using the client inside a job/service scheduled with Polyaxon will have access to the
               token of the user who started the run if the `auth` context is enabled.

    You can access specific low level clients:

    ```python
    >>> client = PolyaxonClient()

    >>> client.projects_v1
    >>> client.runs_v1
    >>> client.auth_v1
    >>> client.users_v1
    >>> client.agents_v1
    >>> client.connections_v1
    >>> client.organizations_v1
    ```

    If you are interacting with a run or with a project, we suggest that you check:
     * [RunClient](/docs/core/python-library/run-client/)
     * [ProjectClient](/docs/core/python-library/project-client/)
    """

    def __init__(self, config=None, token=None):

        self._config = config or settings.CLIENT_CONFIG
        token = token or self._config.token
        if not token:
            self._config.token = settings.AUTH_CONFIG.token
        elif token == NO_AUTH:
            self._config.token = None
        else:
            self._config.token = token

        self._transport = None
        self.api_client = ApiClient(self.config.sdk_config, **self.config.client_header)
        self._projects_v1 = None
        self._runs_v1 = None
        self._auth_v1 = None
        self._users_v1 = None
        self._versions_v1 = None
        self._agents_v1 = None
        self._organizations_v1 = None
        self._connections_v1 = None

    def reset(self):
        self._transport = None
        self._projects_v1 = None
        self._runs_v1 = None
        self._auth_v1 = None
        self._users_v1 = None
        self._versions_v1 = None
        self._agents_v1 = None
        self.api_client = ApiClient(self.config.sdk_config, **self.config.client_header)

    @property
    def config(self):
        return self._config

    @property
    def projects_v1(self):
        if not self._projects_v1:
            self._projects_v1 = polyaxon_sdk.ProjectsV1Api(self.api_client)
        return self._projects_v1

    @property
    def runs_v1(self):
        if not self._runs_v1:
            self._runs_v1 = polyaxon_sdk.RunsV1Api(self.api_client)
        return self._runs_v1

    @property
    def auth_v1(self):
        if not self._auth_v1:
            self._auth_v1 = polyaxon_sdk.AuthV1Api(self.api_client)
        return self._auth_v1

    @property
    def users_v1(self):
        if not self._users_v1:
            self._users_v1 = polyaxon_sdk.UsersV1Api(self.api_client)
        return self._users_v1

    @property
    def versions_v1(self):
        if not self._versions_v1:
            self._versions_v1 = polyaxon_sdk.VersionsV1Api(self.api_client)
        return self._versions_v1

    @property
    def agents_v1(self):
        if not self._agents_v1:
            self._agents_v1 = polyaxon_sdk.AgentsV1Api(self.api_client)
        return self._agents_v1

    @property
    def connections_v1(self):
        if not self._connections_v1:
            self._connections_v1 = polyaxon_sdk.ConnectionsV1Api(self.api_client)
        return self._connections_v1

    @property
    def organizations_v1(self):
        if not self._organizations_v1:
            self._organizations_v1 = polyaxon_sdk.ConnectionsV1Api(self.api_client)
        return self._organizations_v1

    def sanitize_for_serialization(self, value):
        return self.api_client.sanitize_for_serialization(value)



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from kubernetes.client import Configuration

from polyaxon import settings
from polyaxon.exceptions import PolyaxonAgentError
from polyaxon.polypod.mixins import MIXIN_MAPPING, BaseMixin


class BaseSpawner:
    def __init__(
        self,
        namespace: str = None,
        k8s_config: Configuration = None,
        in_cluster: bool = None,
    ):
        if in_cluster is None:
            in_cluster = settings.CLIENT_CONFIG.in_cluster

        if not namespace:
            namespace = settings.CLIENT_CONFIG.namespace

        self.namespace = namespace
        self.in_cluster = in_cluster
        self.k8s_config = k8s_config
        self._k8s_manager = None

    @staticmethod
    def _get_mixin_for_kind(kind: str) -> BaseMixin:
        m = MIXIN_MAPPING.get(kind)
        if not m:
            raise PolyaxonAgentError("Agent received unrecognized kind {}".format(kind))
        return m

    @property
    def k8s_manager(self):
        raise NotImplementedError



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json
import os

from collections import namedtuple
from datetime import datetime
from typing import Dict, List

from marshmallow import fields

from polyaxon.contexts import paths as ctx_paths
from polyaxon.schemas.base import BaseConfig, BaseSchema
from polyaxon.utils.date_utils import path_last_modified
from polyaxon.utils.path_utils import get_files_and_dirs_in_path


class PathData(namedtuple("PathData", "base ts op")):
    pass


class FSWatcherSchema(BaseSchema):
    dir_mapping = fields.Dict(allow_none=True)
    file_mapping = fields.Dict(allow_none=True)

    @staticmethod
    def schema_config():
        return FSWatcherConfig


class FSWatcherConfig(BaseConfig):
    SCHEMA = FSWatcherSchema
    IDENTIFIER = "fswatcher"

    def __init__(self, dir_mapping=None, file_mapping=None, **kwargs):
        self.dir_mapping = dir_mapping
        self.file_mapping = file_mapping

    @staticmethod
    def _datetime_handler(value):
        if isinstance(value, datetime):
            return value.isoformat()
        raise TypeError("Unknown type")

    @classmethod
    def _dump(cls, obj_dict):
        return json.dumps(obj_dict, default=cls._datetime_handler)

    @staticmethod
    def _parse_mapping(mapping: Dict):
        if not mapping:
            return None
        return {
            k: PathData(v[0], datetime.fromisoformat(v[1]), v[2])
            for k, v in mapping.items()
        }

    @staticmethod
    def delete(path: str):
        if os.path.exists(path):
            os.remove(path)

    def get_dir_mapping(self):
        return self._parse_mapping(self.dir_mapping)

    def get_file_mapping(self):
        return self._parse_mapping(self.file_mapping)


class FSWatcher:
    PUT = "put"
    RM = "rm"
    NOOP = ""

    def __init__(self, dir_mapping: Dict = None, file_mapping: Dict = None):
        self._dir_mapping = dir_mapping or {}
        self._file_mapping = file_mapping or {}

    @classmethod
    def read(cls, config_path: str = ctx_paths.CONTEXT_MOUNT_FILE_WATCHER):
        if not os.path.exists(config_path):
            return cls()
        config = FSWatcherConfig.read(config_path)
        return cls(
            dir_mapping=config.get_dir_mapping(), file_mapping=config.get_file_mapping()
        )

    def write(self, config_path: str = ctx_paths.CONTEXT_MOUNT_FILE_WATCHER):
        config = FSWatcherConfig.read(
            {
                "dir_mapping": self._dir_mapping,
                "file_mapping": self._file_mapping,
            }
        )
        return config.write(config_path)

    def _sync_path(self, path: str, base_path: str, mapping: Dict):
        current_ts = path_last_modified(path)
        rel_path = os.path.relpath(path, base_path)
        data = mapping.get(rel_path)
        if data:
            if current_ts > data.ts:
                mapping[rel_path] = PathData(base_path, current_ts, self.PUT)
            else:
                mapping[rel_path] = PathData(base_path, data.ts, self.NOOP)
        else:
            mapping[rel_path] = PathData(base_path, current_ts, self.PUT)
        return mapping

    def sync_file(self, path: str, base_path: str):
        self._file_mapping = self._sync_path(path, base_path, self._file_mapping)

    def sync_dir(self, path: str, base_path: str):
        self._dir_mapping = self._sync_path(path, base_path, self._dir_mapping)

    def init(self):
        self._dir_mapping = {
            p: PathData(d.base, d.ts, self.RM) for p, d in self._dir_mapping.items()
        }
        self._file_mapping = {
            p: PathData(d.base, d.ts, self.RM) for p, d in self._file_mapping.items()
        }

    def sync(self, path: str, exclude: List[str] = None):
        files, dirs = get_files_and_dirs_in_path(
            path, exclude=exclude, collect_dirs=True
        )
        base_path, prefix_path = os.path.split(path)
        for file_path in files:
            self.sync_file(os.path.join(prefix_path, file_path), base_path=base_path)

        for dir_path in dirs:
            self.sync_dir(os.path.join(prefix_path, dir_path), base_path=base_path)

    def _get_mapping_by_op(self, mapping: Dict, op: str):
        return {(p.base, k) for k, p in mapping.items() if p.op == op}

    def _clean_by_op(self, mapping: Dict, op: str):
        return {k: p for k, p in mapping.items() if p.op != op}

    def get_files_to_put(self):
        return self._get_mapping_by_op(self._file_mapping, self.PUT)

    def get_files_to_rm(self):
        results = self._get_mapping_by_op(self._file_mapping, self.RM)
        self._file_mapping = self._clean_by_op(self._file_mapping, self.RM)
        return results

    def get_dirs_to_put(self):
        return self._get_mapping_by_op(self._dir_mapping, self.PUT)

    def get_dirs_to_rm(self):
        results = self._get_mapping_by_op(self._dir_mapping, self.RM)
        self._file_mapping = self._clean_by_op(self._file_mapping, self.RM)
        return results



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict

from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.notifiers.keys import INTEGRATIONS_WEBHOOKS_PAGERDUTY
from polyaxon.notifiers.spec import NotificationSpec
from polyaxon.notifiers.webhook import WebHookNotifier


class PagerDutyWebHookNotifier(WebHookNotifier):
    notification_key = V1ConnectionKind.PAGERDUTY
    name = "PagerDuty WebHook"
    description = "PagerDuty webhooks to send event payload to pagerduty."
    raise_empty_context = True
    config_key = INTEGRATIONS_WEBHOOKS_PAGERDUTY
    validate_keys = ["service_key"]

    @classmethod
    def serialize_notification_to_context(cls, notification: NotificationSpec) -> Dict:
        payload = {
            "event_type": notification.get_title(),
            "description": notification.condition.reason,
            "details": notification.get_details(),
            "incident_key": "trigger",
            "client": "polyaxon",
            "client_url": cls.get_url(notification),
            "contexts": [],
        }

        return payload

    @classmethod
    def _prepare(cls, context: Dict) -> Dict:
        context = super()._prepare(context)

        return {
            "event_type": context.get("event_type"),
            "description": context.get("description"),
            "details": context.get("details"),
            "incident_key": context.get("incident_key"),
            "client": context.get("client", "Polyaxon"),
            "client_url": context.get("client_url", "https://polyaxon.com"),
            "contexts": context.get("contexts"),
        }

    @classmethod
    def _pre_execute_web_hook(cls, data: Dict, config: Dict) -> Dict:
        service_key = config.get("service_key")
        if service_key:
            data["service_key"] = service_key

        return data



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import Dict

from polyaxon.exceptions import HTTP_ERROR_MESSAGES_MAPPING
from polyaxon.utils.formatting import Printer


def handle_cli_error(
    e, message: str = None, http_messages_mapping: Dict = None, sys_exit: bool = False
):
    http_messages_mapping = http_messages_mapping or {}
    if message:
        Printer.print_error(message)
    if hasattr(e, "status"):
        if e.status not in [404, 401, 403]:
            Printer.print_error("Error message: {}.".format(e))
        message = http_messages_mapping.get(
            e.status, HTTP_ERROR_MESSAGES_MAPPING.get(e.status)
        )
        Printer.print_error(message, sys_exit=sys_exit)
    elif hasattr(e, "message"):  # Handling of HTML errors
        if "404" in e.message:
            message = http_messages_mapping.get(
                404, HTTP_ERROR_MESSAGES_MAPPING.get(404)
            )
            Printer.print_error(message, sys_exit=sys_exit)
        elif "401" in e.message:
            message = http_messages_mapping.get(
                401, HTTP_ERROR_MESSAGES_MAPPING.get(401)
            )
            Printer.print_error(message, sys_exit=sys_exit)
        elif "403" in e.message:
            message = http_messages_mapping.get(
                403, HTTP_ERROR_MESSAGES_MAPPING.get(403)
            )
            Printer.print_error(message, sys_exit=sys_exit)
        else:
            Printer.print_error("Error message: {}.".format(e), sys_exit=sys_exit)
    else:
        Printer.print_error("Error message: {}.".format(e), sys_exit=sys_exit)


def handle_command_not_in_ce():
    Printer.print_error(
        "You are running Polyaxon CE which does not support this command!", True
    )



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow import fields

import polyaxon_sdk

from polyaxon.polyflow.params.params import ParamSchema, ParamValueMixin
from polyaxon.schemas.base import BaseCamelSchema, BaseConfig
from polyaxon.schemas.fields.ref_or_obj import RefOrObject


class JoinParamSchema(ParamSchema):
    ref = None

    @staticmethod
    def schema_config():
        return V1JoinParam


class V1JoinParam(BaseConfig, ParamValueMixin, polyaxon_sdk.V1JoinParam):
    SCHEMA = JoinParamSchema
    IDENTIFIER = "join_param"
    REDUCED_ATTRIBUTES = [
        "contextOnly",
        "connection",
        "toInit",
        "toEnv",
    ]

    @property
    def is_literal(self):
        return False

    @property
    def is_ref(self):
        return True

    @property
    def is_template_ref(self):
        return False

    @property
    def is_join_ref(self):
        return True

    @property
    def is_runs_ref(self):
        return False

    @property
    def is_ops_ref(self):
        return False

    @property
    def is_dag_ref(self):
        return False


class JoinSchema(BaseCamelSchema):
    query = fields.Str(required=True)
    sort = fields.Str(allow_none=True)
    limit = RefOrObject(fields.Int(allow_none=True))
    offset = RefOrObject(fields.Int(allow_none=True))
    params = fields.Dict(
        keys=fields.Str(), values=fields.Nested(JoinParamSchema), allow_none=True
    )

    @staticmethod
    def schema_config():
        return V1Join


class V1Join(BaseConfig, polyaxon_sdk.V1Join):
    """Joins allow to query several runs based on a search specification.

    The result of the join will be a list of values based on the results from executing the search.

    A Join corresponds to a valid [query specification](/docs/core/query-language/),
    the result of the search will be used to resolve
    the params defined in the join.

    ```yaml
    >>> joins:
    >>>   - query: "metrics.loss: <0.01"
    >>>     sort: "metrics.loss"
    >>>     limit: 5
    >>>     params:
    >>>       all_param1:
    >>>         value: inputs.param1
    >>>       all_result1:
    >>>         value: outputs.result1
    >>>       all_result2:
    >>>         value: outputs.result2
    >>>       tensorboard_paths:
    >>>         value: artifacts.tensorboard
    >>>         contextOnly: true
    >>>   - query: "metrics.accuracy: >.9"
    >>>     sort: "-metrics.accuracy"
    >>>     params:
    >>>       all_inputs:
    >>>         value: inputs
    >>>       all_outputs:
    >>>         value: outputs
    >>>       run_artifact_paths:
    >>>         value: artifacts.base
    >>>       uuids:
    >>>         value: globals.uuid
    >>>         contextOnly: true
    >>>       files:
    >>>         value: {files: ["subpath/files", "subpath2/file2"]}
    >>>         toInit: true
    ```

    ```python
    >>> joins = [
    >>>     V1Join(
    >>>         query="metrics.loss: <0.01",
    >>>         sort="metrics.loss",
    >>>         limit=5,
    >>>         params={
    >>>             "all_param1": V1JoinParam(value="inputs.param1"),
    >>>             "all_result1": V1JoinParam(value="outputs.result1"),
    >>>             "all_result2": V1JoinParam(value="outputs.result2"),
    >>>             "tensorboard_paths": V1JoinParam(
    >>>                 value="artifacts.tensorboard", context_only=True
    >>>             ),
    >>>         },
    >>>     ),
    >>>     V1Join(
    >>>         query="metrics.accuracy: >.9",
    >>>         sort="-metrics.accuracy",
    >>>         params={
    >>>             "all_inputs": V1JoinParam(value="inputs"),
    >>>             "all_outputs": V1JoinParam(value="outputs"),
    >>>             "run_artifact_paths": V1JoinParam(value="artifacts")
    >>>             "uuids": V1JoinParam(value="globals.uuid"),
    >>>             "artifacts": V1JoinParam(
    >>>                 value={files: ["subpath/files", "subpath2/file2"]},
    >>>                 to_init=True,
    >>>             ),
    >>>         }
    >>>     )
    >>> ]
    ```

    This will instruct Polyaxon to perform 2 searches.
    Each search will expose the params to be used similar to the default
    [params section](/docs/core/specification/params/).
    Polyaxon will validate the params of search against the IO(inputs/outputs) definition.
    Users should make sure that their IO definition specify the `isList: true`,
    unless the type is `artifacts`.

    If a param is based on `contexts`, `inputs`, `outputs`, or `artifacts`
    Polyaxon will turn that param into a list by querying
    that field from all runs in the search result:

    ```python
    >>>  {
    >>>     "all_param1": [val_run_1, val_run_223, val_run_234, ...],
    >>>     ...
    >>> }
    ```

    When the param is of type [ArtifactsType](/docs/core/specification/types/#v1artifactstype),
    all files and dirs will be concatenated in a single list,
    each value will be prefixed with the uuid (run path) of each run in the query result:

    ```python
    >>>  {
    >>>     "artifacts": {
    >>>         "file": [
    >>>             "run_3/subpath/files", "run_3/subpath2/file2",
    >>>             "run_4/subpath/files", "run_4/subpath2/file2",
    >>>             ...
    >>>         ],
    >>>     }
    >>>     ...
    >>> }
    ```

    > **Note**: the difference between using `artifacts.lineage_name`
    > and [ArtifactsType](/docs/core/specification/types/#v1artifactstype),
    > is that the former will only expose the path(s) based on any lineage logged
    > during the runtime, the later is a manual way of selecting specific files and dirs.

    ## Fields

    ### query

    A valid query respecting
    [Polyaxon Query Language](/docs/core/query-language/runs/#query)

    ```yaml
    >>> joins:
    >>>   - query: "metrics.loss: <0.01, project.name: {{ globals.project_name}}, kind: job"
    ```

    ### sort

    A valid sort respecting
    [Polyaxon Query Language](/docs/core/query-language/runs/#sort)

    ```yaml
    >>> joins:
    >>>   - sort: "created_at, -metrics.loss"
    ```

    ### limit

    The maximum number of runs to join based on the query/sort condition.

    > **Note**: at the moment we have a hard limit, `5000`, on the number of upstream runs to join.

    ```yaml
    >>> joins:
    >>>   - limit: "10"
    ```

    ### offset

    ```yaml
    >>> joins:
    >>>   - offset: "100"
    ```

    An optional integer used for pagination.

    ### params

    Similar to the default [params specification](/docs/core/specification/params/)
    with the exception that it does not accept the `ref` key.
    The reference is generated automatically based on the search performed by Polyaxon.

    The fields supported: `value`, `context_only`, `connection`, `to_init`
    """

    SCHEMA = JoinSchema
    IDENTIFIER = "join"
    REDUCED_ATTRIBUTES = [
        "sort",
        "limit",
        "offset",
    ]



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
import sys

from functools import wraps
from typing import List, Union

from polyaxon.env_vars.keys import EV_KEYS_DEBUG, EV_KEYS_LOG_LEVEL

logger = logging.getLogger("polyaxon.cli")


def configure_logger(verbose):
    # DO NOT MOVE OUTSIDE OF THE FUNCTION!
    from polyaxon import settings
    from polyaxon.plugins.sentry import set_raven_client

    if verbose or settings.CLIENT_CONFIG.debug or os.environ.get(EV_KEYS_DEBUG, False):
        log_level = logging.DEBUG
        settings.CLIENT_CONFIG.debug = True
    else:
        if not settings.CLIENT_CONFIG.disable_errors_reporting:
            set_raven_client()
        log_level = (
            logging.DEBUG
            if os.environ.get(EV_KEYS_LOG_LEVEL) in ["debug", "DEBUG"]
            else logging.INFO
        )
        if settings.CLIENT_CONFIG.log_level:
            try:
                log_level = logging.getLevelName(settings.CLIENT_CONFIG.log_level)
            except Exception:  # noqa
                pass
    logging.basicConfig(format="%(message)s", level=log_level, stream=sys.stdout)


def clean_outputs(fn):
    """Decorator for CLI with Sentry client handling.
    see https://github.com/getsentry/sentry-python/issues/862#issuecomment-712697356
    """

    @wraps(fn)
    def clean_outputs_wrapper(*args, **kwargs):
        from polyaxon import settings

        cli_config = settings.CLI_CONFIG
        if cli_config and cli_config.log_handler and cli_config.log_handler.dsn:
            import sentry_sdk

            try:
                sentry_sdk.flush()
                return fn(*args, **kwargs)
            except Exception as e:
                sentry_sdk.capture_exception(e)
                sentry_sdk.flush()
                raise e
            finally:
                sentry_sdk.flush()
        else:
            return fn(*args, **kwargs)

    return clean_outputs_wrapper


def not_in_ce(fn):
    """Decorator to show an error when a command not available in CE"""

    @wraps(fn)
    def not_in_ce_wrapper(*args, **kwargs):
        from polyaxon import settings
        from polyaxon.cli.errors import handle_command_not_in_ce

        if not settings.CLI_CONFIG or settings.CLI_CONFIG.is_community:
            handle_command_not_in_ce()
        return fn(*args, **kwargs)

    return not_in_ce_wrapper


def reconfigure_loggers(
    logger_names: List[str],
    log_level: Union[int, str] = logging.WARNING,
    disable: bool = True,
):
    for logger_name in logger_names:
        logging.getLogger(logger_name).setLevel(log_level)
        if disable:
            logging.getLogger(logger_name).disabled = True



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# To keep backwards compatibility

from typing import List, Optional

from polyaxon.client import RunClient
from traceml import tracking
from traceml.tracking import *

TRACKING_RUN = None


def init(
    owner: str = None,
    project: str = None,
    run_uuid: str = None,
    client: RunClient = None,
    track_code: bool = True,
    track_env: bool = True,
    refresh_data: bool = False,
    artifacts_path: str = None,
    collect_artifacts: str = None,
    collect_resources: str = None,
    is_offline: bool = None,
    is_new: bool = None,
    name: str = None,
    description: str = None,
    tags: List[str] = None,
) -> Optional[Run]:
    global TRACKING_RUN

    TRACKING_RUN = tracking.init(
        owner=owner,
        project=project,
        run_uuid=run_uuid,
        client=client,
        track_code=track_code,
        track_env=track_env,
        refresh_data=refresh_data,
        artifacts_path=artifacts_path,
        collect_artifacts=collect_artifacts,
        collect_resources=collect_resources,
        is_offline=is_offline,
        is_new=is_new,
        name=name,
        description=description,
        tags=tags,
    )
    return TRACKING_RUN


def get_or_create_run(tracking_run: Run = None) -> Optional[Run]:
    global TRACKING_RUN

    TRACKING_RUN = tracking.get_or_create_run(tracking_run)
    return TRACKING_RUN



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Iterable, List

from polyaxon.schemas.types import V1ConnectionType, V1K8sResourceType


def get_requested_secrets(
    secrets: Iterable[V1K8sResourceType], connections: Iterable[V1ConnectionType]
) -> List[V1K8sResourceType]:
    secrets = secrets or []
    connections = connections or []
    # Create a set of all secrets:
    #   * secrets request by non managed stores
    #   * secrets requested directly by the user
    requested_secrets = [secret for secret in secrets if secret.is_requested]
    secret_ids = {s.name for s in requested_secrets}
    for connection in connections:
        secret = connection.get_secret()
        if secret and secret.name not in secret_ids:
            secret_ids.add(secret.name)
            requested_secrets.append(secret)

    return requested_secrets


def get_requested_config_maps(
    config_maps: Iterable[V1K8sResourceType], connections: Iterable[V1ConnectionType]
) -> List[V1K8sResourceType]:
    config_maps = config_maps or []
    connections = connections or []
    # Create a set of all config_maps:
    #   * secrets request by non managed stores
    #   * secrets requested directly by the user
    requested_config_maps = [
        config_map for config_map in config_maps if config_map.is_requested
    ]
    config_map_ids = {s.name for s in requested_config_maps}
    for connection in connections:
        config_map = connection.get_config_map()
        if config_map and config_map.name not in config_map_ids:
            config_map_ids.add(config_map.name)
            requested_config_maps.append(config_map)

    return requested_config_maps



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import signal
import threading

from contextlib import contextmanager
from typing import Generator

from polyaxon.logger import logger


def get_pool_workers():
    return min(32, (os.cpu_count() or 1) + 4)


def get_core_workers(per_core: int, max_value: int = 8):
    count = per_core * (os.cpu_count() or 1) + 1
    return max(count, max_value) if max_value else count


@contextmanager
def exit_context() -> Generator:
    exit_event = threading.Event()

    def _exit_handler(*args, **kwargs) -> None:
        logger.info("Keyboard Interrupt received, exiting pool.")
        exit_event.set()

    original = signal.getsignal(signal.SIGINT)
    try:
        signal.signal(signal.SIGINT, _exit_handler)
        yield exit_event
    except SystemExit:
        pass
    finally:
        signal.signal(signal.SIGINT, original)


def get_wait(current: int) -> float:
    intervals = [0.25, 0.5, 1.0, 2.0, 4.0, 8.0]

    if current >= 5:
        current = 5
    return intervals[current]



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, Iterable, Optional

from polyaxon import pkg
from polyaxon.polyflow import V1CompiledOperation, V1KFReplica, V1Plugins, V1XGBoostJob
from polyaxon.polypod.compiler.converters import BaseConverter
from polyaxon.polypod.compiler.converters.base import PlatformConverterMixin
from polyaxon.polypod.custom_resources import get_xgb_job_custom_resource
from polyaxon.polypod.mixins import XGBoostJobMixin
from polyaxon.polypod.specs.contexts import PluginsContextsSpec
from polyaxon.polypod.specs.replica import ReplicaSpec
from polyaxon.schemas.types import V1ConnectionType, V1K8sResourceType


class XGBoostJobConverter(XGBoostJobMixin, BaseConverter):
    def get_resource(
        self,
        compiled_operation: V1CompiledOperation,
        artifacts_store: V1ConnectionType,
        connection_by_names: Dict[str, V1ConnectionType],
        secrets: Optional[Iterable[V1K8sResourceType]],
        config_maps: Optional[Iterable[V1K8sResourceType]],
        default_sa: str = None,
        default_auth: bool = False,
    ) -> Dict:
        job = compiled_operation.run  # type: V1XGBoostJob

        def _get_replica(replica: Optional[V1KFReplica]) -> Optional[ReplicaSpec]:
            if not replica:
                return None
            return self.get_replica_resource(
                plugins=plugins,
                contexts=contexts,
                environment=replica.environment,
                volumes=replica.volumes or [],
                init=replica.init or [],
                sidecars=replica.sidecars or [],
                container=replica.container,
                artifacts_store=artifacts_store,
                connections=replica.connections or [],
                connection_by_names=connection_by_names,
                secrets=secrets,
                config_maps=config_maps,
                kv_env_vars=kv_env_vars,
                default_sa=default_sa,
                num_replicas=replica.replicas,
            )

        kv_env_vars = compiled_operation.get_env_io()
        plugins = compiled_operation.plugins or V1Plugins()
        contexts = PluginsContextsSpec.from_config(plugins, default_auth=default_auth)
        master = _get_replica(job.master)
        worker = _get_replica(job.worker)
        labels = self.get_labels(version=pkg.VERSION, labels={})

        return get_xgb_job_custom_resource(
            namespace=self.namespace,
            resource_name=self.resource_name,
            master=master,
            worker=worker,
            termination=compiled_operation.termination,
            collect_logs=contexts.collect_logs,
            clean_pod_policy=job.clean_pod_policy,
            scheduling_policy=job.scheduling_policy,
            sync_statuses=contexts.sync_statuses,
            notifications=plugins.notifications,
            labels=labels,
            annotations=self.annotations,
        )


class PlatformXGBoostJobConverter(PlatformConverterMixin, XGBoostJobConverter):
    pass



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow import EXCLUDE, fields

from polyaxon.k8s import k8s_schemas
from polyaxon.schemas.base import BaseCamelSchema, BaseConfig
from polyaxon.schemas.fields.swagger import SwaggerField


class BaseServiceSchema(BaseCamelSchema):
    image = fields.Str(allow_none=True)
    image_tag = fields.Str(allow_none=True)
    image_pull_policy = fields.Str(allow_none=True)
    resources = SwaggerField(cls=k8s_schemas.V1ResourceRequirements, allow_none=True)
    node_selector = fields.Dict(allow_none=True)
    affinity = SwaggerField(cls=k8s_schemas.V1Affinity, allow_none=True)
    tolerations = fields.List(
        SwaggerField(cls=k8s_schemas.V1Toleration), allow_none=True
    )
    image_pull_secrets = fields.List(fields.Str(), allow_none=True)

    class Meta:
        unknown = EXCLUDE

    @staticmethod
    def schema_config():
        return BaseServiceConfig


class BaseServiceConfig(BaseConfig):
    SCHEMA = BaseServiceSchema
    REDUCED_ATTRIBUTES = [
        "image",
        "imageTag",
        "imagePullPolicy",
        "resources",
        "nodeSelector",
        "affinity",
        "tolerations",
        "imagePullSecrets",
    ]

    def __init__(
        self,
        image=None,
        image_tag=None,
        image_pull_policy=None,
        resources=None,
        node_selector=None,
        affinity=None,
        tolerations=None,
        image_pull_secrets=None,
    ):
        self.image = image
        self.image_tag = image_tag
        self.image_pull_policy = image_pull_policy
        self.resources = resources
        self.node_selector = node_selector
        self.affinity = affinity
        self.tolerations = tolerations
        self.image_pull_secrets = image_pull_secrets



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

OWNER_NAME = "owner_name"
PROJECT_NAME = "project_name"
PROJECT_UUID = "project_uuid"
PROJECT_UNIQUE_NAME = "project_unique_name"
CONTEXT_PATH = "context_path"
ARTIFACTS_PATH = "artifacts_path"
ITERATION = "iteration"
NAMESPACE = "namespace"
BASE = "base"
STORE_PATH = "store_path"

RUN_ARTIFACTS_PATH = "run_artifacts_path"
RUN_OUTPUTS_PATH = "run_outputs_path"
RUN_INFO = "run_info"
STATUS = "status"
CONDITION = "condition"
NAME = "name"
UUID = "uuid"
CREATED_AT = "created_at"
COMPILED_AT = "compiled_at"
SCHEDULE_AT = "schedule_at"
STARTED_AT = "started_at"
FINISHED_AT = "finished_at"
DURATION = "duration"
CLONING_KIND = "cloning_kind"
ORIGINAL_UUID = "original_uuid"
IS_INDEPENDENT = "is_independent"



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from typing import Dict

from marshmallow import ValidationError

from polyaxon.polyflow.run.cleaner import V1CleanerJob
from polyaxon.polyflow.run.dag import V1Dag
from polyaxon.polyflow.run.dask import V1Dask
from polyaxon.polyflow.run.job import V1Job
from polyaxon.polyflow.run.kinds import V1RunKind
from polyaxon.polyflow.run.kubeflow.mpi_job import V1MPIJob
from polyaxon.polyflow.run.kubeflow.mx_job import V1MXJob
from polyaxon.polyflow.run.kubeflow.pytorch_job import V1PytorchJob
from polyaxon.polyflow.run.kubeflow.replica import V1KFReplica
from polyaxon.polyflow.run.kubeflow.tf_job import V1TFJob
from polyaxon.polyflow.run.kubeflow.xgboost_job import V1XGBoostJob
from polyaxon.polyflow.run.notifier import V1NotifierJob
from polyaxon.polyflow.run.service import V1Service
from polyaxon.polyflow.run.spark.replica import V1SparkReplica
from polyaxon.polyflow.run.spark.spark import V1Spark
from polyaxon.polyflow.run.tuner import V1TunerJob


def validate_run_patch(run_patch: Dict, kind: V1RunKind.allowable_values):
    if kind == V1RunKind.JOB:
        patch = V1Job.from_dict(run_patch)
    elif kind == V1RunKind.SERVICE:
        patch = V1Service.from_dict(run_patch)
    elif kind == V1RunKind.DAG:
        patch = V1Dag.from_dict(run_patch)
    elif kind == V1RunKind.MPIJOB:
        try:
            patch = V1MPIJob.from_dict(run_patch)
        except ValidationError:
            patch = V1KFReplica.from_dict(run_patch)
    elif kind == V1RunKind.PYTORCHJOB:
        try:
            patch = V1PytorchJob.from_dict(run_patch)
        except ValidationError:
            patch = V1KFReplica.from_dict(run_patch)
    elif kind == V1RunKind.TFJOB:
        try:
            patch = V1TFJob.from_dict(run_patch)
        except ValidationError:
            patch = V1KFReplica.from_dict(run_patch)
    elif kind == V1RunKind.MXJOB:
        try:
            patch = V1MXJob.from_dict(run_patch)
        except ValidationError:
            patch = V1KFReplica.from_dict(run_patch)
    elif kind == V1RunKind.XGBJOB:
        try:
            patch = V1XGBoostJob.from_dict(run_patch)
        except ValidationError:
            patch = V1KFReplica.from_dict(run_patch)
    elif kind == V1RunKind.SPARK:
        try:
            patch = V1Spark.from_dict(run_patch)
        except ValidationError:
            patch = V1SparkReplica.from_dict(run_patch)
    elif kind == V1RunKind.DASK:
        patch = V1Dask.from_dict(run_patch)
    elif kind == V1RunKind.NOTIFIER:
        patch = V1NotifierJob.from_dict(run_patch)
    elif kind == V1RunKind.TUNER:
        patch = V1TunerJob.from_dict(run_patch)
    elif kind == V1RunKind.CLEANER:
        patch = V1CleanerJob.from_dict(run_patch)
    else:
        raise ValidationError("runPatch cannot be validate without a supported kind.")

    return patch



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import List, Optional

from polyaxon.auxiliaries import V1PolyaxonSidecarContainer
from polyaxon.exceptions import PolypodException
from polyaxon.k8s import k8s_schemas
from polyaxon.polypod.common.containers import patch_container
from polyaxon.polypod.common.env_vars import (
    get_connection_env_var,
    get_env_from_config_map,
    get_env_from_secret,
    get_items_from_config_map,
    get_items_from_secret,
)
from polyaxon.polypod.common.mounts import (
    get_mount_from_resource,
    get_mount_from_store,
    get_mounts,
)
from polyaxon.polypod.sidecar.env_vars import get_sidecar_env_vars
from polyaxon.polypod.specs.contexts import PluginsContextsSpec
from polyaxon.schemas.types import V1ConnectionType
from polyaxon.utils.list_utils import to_list

SIDECAR_CONTAINER = "polyaxon-sidecar"


def get_sidecar_args(
    container_id: str, sleep_interval: int, sync_interval: int, monitor_logs: bool
) -> List[str]:
    args = [
        "--container-id={}".format(container_id),
        "--sleep-interval={}".format(sleep_interval),
        "--sync-interval={}".format(sync_interval),
    ]
    if monitor_logs:
        args.append("--monitor-logs")
    return args


def get_sidecar_container(
    container_id: str,
    polyaxon_sidecar: V1PolyaxonSidecarContainer,
    env: List[k8s_schemas.V1EnvVar],
    artifacts_store: V1ConnectionType,
    contexts: PluginsContextsSpec,
    run_path: Optional[str],
) -> Optional[k8s_schemas.V1Container]:

    if artifacts_store and not contexts:
        raise PolypodException(
            "Logs/artifacts store was passed and contexts was not passed."
        )

    has_artifacts = artifacts_store and contexts.collect_artifacts
    has_logs = artifacts_store and contexts.collect_logs

    if not has_logs and not has_artifacts:
        # No sidecar
        return None

    if (has_artifacts or has_logs) and not run_path:
        raise PolypodException("Logs store/outputs store must have a run_path.")

    env = get_sidecar_env_vars(
        env_vars=env,
        container_id=container_id,
        artifacts_store_name=artifacts_store.name,
    )

    volume_mounts = get_mounts(
        use_auth_context=contexts.auth,
        use_artifacts_context=has_artifacts,
        use_docker_context=False,
        use_shm_context=False,
    )

    sleep_interval = polyaxon_sidecar.sleep_interval
    sync_interval = polyaxon_sidecar.sync_interval
    monitor_logs = polyaxon_sidecar.monitor_logs
    if contexts and contexts.sidecar:
        if contexts.sidecar.sleep_interval:
            sleep_interval = contexts.sidecar.sleep_interval
        if contexts.sidecar.sync_interval:
            sync_interval = contexts.sidecar.sync_interval
        if contexts.sidecar.monitor_logs:
            monitor_logs = contexts.sidecar.monitor_logs
    sidecar_args = get_sidecar_args(
        container_id=container_id,
        sleep_interval=sleep_interval,
        sync_interval=sync_interval,
        monitor_logs=monitor_logs,
    )

    env_from = []

    secret = None
    if artifacts_store.is_bucket:
        secret = artifacts_store.get_secret()
        volume_mounts += to_list(
            get_mount_from_resource(resource=secret), check_none=True
        )
        env += to_list(get_items_from_secret(secret=secret), check_none=True)
        env_from += to_list(get_env_from_secret(secret=secret), check_none=True)

        config_map = artifacts_store.get_config_map()
        volume_mounts += to_list(
            get_mount_from_resource(resource=config_map), check_none=True
        )
        env += to_list(
            get_items_from_config_map(config_map=config_map), check_none=True
        )
        env_from += to_list(
            get_env_from_config_map(config_map=config_map), check_none=True
        )
    else:
        volume_mounts += to_list(
            get_mount_from_store(store=artifacts_store), check_none=True
        )
    env += to_list(
        get_connection_env_var(connection=artifacts_store, secret=secret),
        check_none=True,
    )

    container = k8s_schemas.V1Container(
        name=SIDECAR_CONTAINER,
        image=polyaxon_sidecar.get_image(),
        image_pull_policy=polyaxon_sidecar.image_pull_policy,
        command=["polyaxon", "sidecar"],
        args=sidecar_args,
        env=env,
        env_from=env_from,
        resources=polyaxon_sidecar.get_resources(),
        volume_mounts=volume_mounts,
    )

    return patch_container(container)



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict

from polyaxon.polyflow import V1CompiledOperation
from polyaxon.polypod.compiler.contexts.base import BaseContextsManager
from polyaxon.schemas.types import V1ConnectionType


class JobContextsManager(BaseContextsManager):
    @classmethod
    def resolve(
        cls,
        namespace: str,
        owner_name: str,
        project_name: str,
        run_uuid: str,
        contexts: Dict,
        compiled_operation: V1CompiledOperation,
        connection_by_names: Dict[str, V1ConnectionType],
    ) -> Dict:
        return cls._resolver_replica(
            contexts=contexts,
            init=compiled_operation.run.init,
            connections=compiled_operation.run.connections,
            connection_by_names=connection_by_names,
        )



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from urllib.parse import urlparse


def get_service_url(host: str, port: int):
    if port == 80:
        return "http://{}".format(host)
    if port == 443:
        return "https://{}".format(host)
    return "http://{}:{}".format(host, port)


def has_https(url: str):
    return "https" in url


def get_ssl_server_name(url: str):
    if has_https(url):
        return "proxy_ssl_server_name on;"
    return ""


def get_header_host(url: str):
    if has_https(url):
        return "proxy_set_header Host {};".format(urlparse(url).netloc.split(":")[0])
    return "proxy_set_header Host $http_host;"


def get_service_proxy(port: int = 8443):
    return "https://127.0.0.1:{}".format(port)



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import asyncio
import os

from typing import List

from polyaxon.contexts import paths as ctx_paths
from polyaxon.fs.async_manager import ensure_async_execution
from polyaxon.fs.types import FSSystem
from polyaxon.fs.watcher import FSWatcher
from polyaxon.logger import logger


async def sync_fs(
    fs: FSSystem,
    fw: FSWatcher,
    store_base_path: str,
):
    def get_store_path(subpath: str):
        return os.path.join(store_base_path, subpath)

    rm_files = fw.get_files_to_rm()
    logger.debug("rm_files {}".format(rm_files))
    await asyncio.gather(
        *[
            ensure_async_execution(
                fs=fs,
                fct="rm_file",
                is_async=fs.async_impl,
                path=get_store_path(subpath),
                recursive=False,
            )
            for (_, subpath) in rm_files
        ],
        return_exceptions=True
    )
    rm_dirs = fw.get_dirs_to_rm()
    logger.debug("rm_dirs {}".format(rm_dirs))
    await asyncio.gather(
        *[
            ensure_async_execution(
                fs=fs,
                fct="rm",
                is_async=fs.async_impl,
                path=get_store_path(subpath),
                recursive=True,
            )
            for (_, subpath) in rm_dirs
        ],
        return_exceptions=True
    )
    put_files = fw.get_files_to_put()
    logger.debug("put_files {}".format(put_files))
    await asyncio.gather(
        *[
            ensure_async_execution(
                fs=fs,
                fct="put",
                is_async=fs.async_impl,
                lpath=os.path.join(r_base_path, subpath),
                rpath=get_store_path(subpath),
                recursive=False,
            )
            for (r_base_path, subpath) in put_files
        ],
        return_exceptions=True
    )


async def sync_artifacts(
    fs: FSSystem,
    fw: FSWatcher,
    store_path: str,
    run_uuid: str,
    exclude: List[str] = None,
):
    fw.init()
    path_from = ctx_paths.CONTEXT_MOUNT_ARTIFACTS_FORMAT.format(run_uuid)
    fw.sync(path_from, exclude=exclude)

    # Check if this run has triggered some related run paths
    if os.path.exists(ctx_paths.CONTEXT_MOUNT_ARTIFACTS_RELATED):
        for sub_path in os.listdir(ctx_paths.CONTEXT_MOUNT_ARTIFACTS_RELATED):
            # check if there's a path to sync
            path_from = ctx_paths.CONTEXT_MOUNT_ARTIFACTS_RELATED_FORMAT.format(
                sub_path
            )
            fw.sync(path_from, exclude=exclude)

    await sync_fs(
        fs=fs,
        fw=fw,
        store_base_path=store_path,
    )



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.config_reader.spec import ConfigSpec
from polyaxon.parser import parser
from polyaxon.parser.constants import NO_VALUE_FOUND
from polyaxon.schemas.types import V1AuthType, V1UriType


class ConfigManager:
    def __init__(self, **data):
        self._data = data
        self._requested_keys = set()
        self._secret_keys = set()
        self._local_keys = set()

    @classmethod
    def read_configs(cls, config_values):  # pylint:disable=redefined-outer-name
        config = ConfigSpec.read_from(
            config_values
        )  # pylint:disable=redefined-outer-name
        return cls(**config) if config else None

    def keys_startswith(self, term):
        return [k for k in self._data if k.startswith(term)]

    def keys_endswith(self, term):
        return [k for k in self._data if k.endswith(term)]

    def has_key(self, key):
        return key in self._data

    @property
    def data(self):
        return self._data

    @property
    def requested_keys(self):
        return self._requested_keys

    @property
    def secret_keys(self):
        return self._secret_keys

    @property
    def local_keys(self):
        return self._local_keys

    def get_requested_data(
        self, include_secrets=False, include_locals=False, to_str=False
    ):
        data = {}
        for key in self._requested_keys:
            if not include_secrets and key in self._secret_keys:
                continue
            if not include_locals and key in self._local_keys:
                continue
            value = self._data[key]
            data[key] = "{}".format(value) if to_str else value
        return data

    def get_int(
        self,
        key,
        is_list=False,
        is_optional=False,
        is_secret=False,
        is_local=False,
        default=None,
        options=None,
    ):
        """
        Get the value corresponding to the key and converts it to `int`/`list(int)`.

        Args:
            key: the dict key.
            is_list: If this is one element or a list of elements.
            is_optional: To raise an error if key was not found.
            is_secret: If the key is a secret.
            is_local: If the key is a local to this service.
            default: default value if is_optional is True.
            options: list/tuple if provided, the value must be one of these values.

        Returns:
             `int`: value corresponding to the key.
        """
        return self._get(
            key=key,
            parser_fct=parser.get_int,
            is_list=is_list,
            is_optional=is_optional,
            is_secret=is_secret,
            is_local=is_local,
            default=default,
            options=options,
        )

    def get_float(
        self,
        key,
        is_list=False,
        is_optional=False,
        is_secret=False,
        is_local=False,
        default=None,
        options=None,
    ):
        """
        Get the value corresponding to the key and converts it to `float`/`list(float)`.

        Args:
            key: the dict key.
            is_list: If this is one element or a list of elements.
            is_optional: To raise an error if key was not found.
            is_secret: If the key is a secret.
            is_local: If the key is a local to this service.
            default: default value if is_optional is True.
            options: list/tuple if provided, the value must be one of these values.

        Returns:
             `float`: value corresponding to the key.
        """
        return self._get(
            key=key,
            parser_fct=parser.get_float,
            is_list=is_list,
            is_optional=is_optional,
            is_secret=is_secret,
            is_local=is_local,
            default=default,
            options=options,
        )

    def get_boolean(
        self,
        key,
        is_list=False,
        is_optional=False,
        is_secret=False,
        is_local=False,
        default=None,
        options=None,
    ):
        """
        Get the value corresponding to the key and converts it to `bool`/`list(str)`.

        Args:
            key: the dict key.
            is_list: If this is one element or a list of elements.
            is_optional: To raise an error if key was not found.
            is_secret: If the key is a secret.
            is_local: If the key is a local to this service.
            default: default value if is_optional is True.
            options: list/tuple if provided, the value must be one of these values.

        Returns:
            `bool`: value corresponding to the key.
        """
        return self._get(
            key=key,
            parser_fct=parser.get_boolean,
            is_list=is_list,
            is_optional=is_optional,
            is_secret=is_secret,
            is_local=is_local,
            default=default,
            options=options,
        )

    def get_string(
        self,
        key,
        is_list=False,
        is_optional=False,
        is_secret=False,
        is_local=False,
        default=None,
        options=None,
    ):
        """
        Get the value corresponding to the key and converts it to `str`/`list(str)`.

        Args:
            key: the dict key.
            is_list: If this is one element or a list of elements.
            is_optional: To raise an error if key was not found.
            is_secret: If the key is a secret.
            is_local: If the key is a local to this service.
            default: default value if is_optional is True.
            options: list/tuple if provided, the value must be one of these values.

        Returns:
            `str`: value corresponding to the key.
        """
        return self._get(
            key=key,
            parser_fct=parser.get_string,
            is_list=is_list,
            is_optional=is_optional,
            is_secret=is_secret,
            is_local=is_local,
            default=default,
            options=options,
            strong_type=True,
        )

    def get_dict(
        self,
        key,
        is_list=False,
        is_optional=False,
        is_secret=False,
        is_local=False,
        default=None,
        options=None,
    ):
        """
        Get the value corresponding to the key and converts it to `dict`.

        Args:
            key: the dict key.
            is_list: If this is one element or a list of elements.
            is_optional: To raise an error if key was not found.
            is_secret: If the key is a secret.
            is_local: If the key is a local to this service.
            default: default value if is_optional is True.
            options: list/tuple if provided, the value must be one of these values.

        Returns:
            `str`: value corresponding to the key.
        """

        return self._get(
            key=key,
            parser_fct=parser.get_dict,
            is_list=is_list,
            is_optional=is_optional,
            is_secret=is_secret,
            is_local=is_local,
            default=default,
            options=options,
        )

    def get_dict_of_dicts(
        self,
        key,
        is_optional=False,
        is_secret=False,
        is_local=False,
        default=None,
        options=None,
    ):
        """
        Get the value corresponding to the key and converts it to `dict`.

        Add an extra validation that all keys have a dict as values.

        Args:
            key: the dict key.
            is_optional: To raise an error if key was not found.
            is_secret: If the key is a secret.
            is_local: If the key is a local to this service.
            default: default value if is_optional is True.
            options: list/tuple if provided, the value must be one of these values.

        Returns:
            `str`: value corresponding to the key.
        """
        return self._get(
            key=key,
            parser_fct=parser.get_dict_of_dicts,
            is_list=None,
            is_optional=is_optional,
            is_secret=is_secret,
            is_local=is_local,
            default=default,
            options=options,
        )

    def get_uri(
        self,
        key,
        is_list=False,
        is_optional=False,
        is_secret=False,
        is_local=False,
        default=None,
        options=None,
    ) -> V1UriType:
        """
        Get the value corresponding to the key and converts it to `V1UriType`.

        Args
            key: the dict key.
            is_list: If this is one element or a list of elements.
            is_optional: To raise an error if key was not found.
            is_secret: If the key is a secret.
            is_local: If the key is a local to this service.
            default: default value if is_optional is True.
            options: list/tuple if provided, the value must be one of these values.

        Returns:
             `str`: value corresponding to the key.
        """
        return self._get(
            key=key,
            parser_fct=parser.get_uri,
            is_list=is_list,
            is_optional=is_optional,
            is_secret=is_secret,
            is_local=is_local,
            default=default,
            options=options,
        )

    def get_auth(
        self,
        key,
        is_list=False,
        is_optional=False,
        is_secret=False,
        is_local=False,
        default=None,
        options=None,
    ) -> V1AuthType:
        """
        Get the value corresponding to the key and converts it to `V1AuthType`.

        Args
            key: the dict key.
            is_list: If this is one element or a list of elements.
            is_optional: To raise an error if key was not found.
            is_secret: If the key is a secret.
            is_local: If the key is a local to this service.
            default: default value if is_optional is True.
            options: list/tuple if provided, the value must be one of these values.

        Returns:
             `str`: value corresponding to the key.
        """
        return self._get(
            key=key,
            parser_fct=parser.get_auth,
            is_list=is_list,
            is_optional=is_optional,
            is_secret=is_secret,
            is_local=is_local,
            default=default,
            options=options,
        )

    def get_list(
        self,
        key,
        is_optional=False,
        is_secret=False,
        is_local=False,
        default=None,
        options=None,
    ):
        """
        Get the value corresponding to the key and converts comma separated values to a list.

        Args:
            key: the dict key.
            is_optional: To raise an error if key was not found.
            is_secret: If the key is a secret.
            is_local: If the key is a local to this service.
            default: default value if is_optional is True.
            options: list/tuple if provided, the value must be one of these values.

        Returns:
             `str`: value corresponding to the key.
        """

        return self._get(
            key=key,
            parser_fct=parser.get_list,
            is_list=None,
            is_optional=is_optional,
            is_secret=is_secret,
            is_local=is_local,
            default=default,
            options=options,
        )

    def _get(
        self,
        key,
        parser_fct,
        is_list,
        is_optional,
        is_secret,
        is_local,
        default,
        options,
        **kwargs,
    ):
        """
        Get key from the dictionary made out of the configs passed.

        Args:
            key: the dict key.

        Returns:
             The corresponding value of the key if found.

        Raises:
            KeyError
        """
        value = self._data.get(key, NO_VALUE_FOUND)
        parsed_value = parser_fct(
            key=key,
            value=value,
            is_list=is_list,
            is_optional=is_optional,
            default=default,
            options=options,
            **kwargs,
        )
        self._add_key(key, is_secret=is_secret, is_local=is_local)
        return parsed_value

    def _add_key(self, key, is_secret=False, is_local=False):
        self._requested_keys.add(key)
        if is_secret:
            self._secret_keys.add(key)
        if is_local:
            self._local_keys.add(key)



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class ManagerInterface:
    def __init__(self):
        self._state = {}

    @property
    def state(self):
        return self._state

    def _get_state_data(self, obj):
        raise NotImplementedError

    def subscribe(self, obj):
        key, value = self._get_state_data(obj)
        self._subscribe(key=key, value=value)

    def _subscribe(self, key, value):
        if key in self.state:
            assert self.state[key] == value
        else:
            self.state[key] = value

    def knows(self, key):
        return key in self.state

    def get(self, key):
        return self.state.get(key)

    @property
    def keys(self):
        return self.state.keys()

    @property
    def values(self):
        return self.state.values()

    @property
    def items(self):
        return self.state.items()



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.constants.globals import UNKNOWN


class EventTypes:
    ADDED = "ADDED"
    MODIFIED = "MODIFIED"
    DELETED = "DELETED"
    ERROR = "ERROR"


class PodConditions:
    READY = "Ready"
    INITIALIZED = "Initialized"
    SCHEDULED = "PodScheduled"
    UNSCHEDULABLE = "Unschedulable"

    VALUES = [READY, INITIALIZED, SCHEDULED]


class PodLifeCycle:
    CONTAINER_CREATING = "ContainerCreating"
    PENDING = "Pending"
    RUNNING = "Running"
    SUCCEEDED = "Succeeded"
    FAILED = "Failed"
    UNKNOWN = UNKNOWN

    DONE_STATUS = [FAILED, SUCCEEDED]



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os

from polyaxon.env_vars.keys import EV_KEYS_SERVICE


class PolyaxonServices:  # noqa
    PLATFORM = "platform"
    AUTH = "auth"
    UI = "ui"
    API = "api"
    GATEWAY = "gateway"
    STREAMS = "streams"
    SANDBOX = "sandbox"
    CLI = "cli"
    INITIALIZER = "initializer"
    INIT = "init"
    SIDECAR = "sidecar"
    RUNNER = "runner"
    AGENT = "agent"
    OPERATOR = "operator"
    BILLING = "billing"
    HP_SEARCH = "hp_search"
    EVENTS_HANDLER = "events-handlers"

    VALUES = {
        PLATFORM,
        CLI,
        UI,
        API,
        GATEWAY,
        STREAMS,
        SANDBOX,
        AUTH,
        INITIALIZER,
        SIDECAR,
        RUNNER,
        AGENT,
        OPERATOR,
        BILLING,
        HP_SEARCH,
        EVENTS_HANDLER,
    }
    AGENT_VALUES = [PLATFORM, CLI, UI, OPERATOR, AGENT, INITIALIZER, SIDECAR]
    SERVICE = None

    @classmethod
    def set_service_name(cls, value: str = None):
        cls.SERVICE = value or os.environ.get(EV_KEYS_SERVICE)

    @classmethod
    def is_agent(cls, value: str = None):
        return cls.AGENT == (value or cls.SERVICE)

    @classmethod
    def is_sandbox(cls, value: str = None):
        return cls.SANDBOX == (value or cls.SERVICE)

    @classmethod
    def is_hp_search(cls, value: str = None):
        return cls.HP_SEARCH == (value or cls.SERVICE)

    @classmethod
    def is_init(cls, value: str = None):
        return (value or cls.SERVICE) in {cls.INIT, cls.INITIALIZER}

    @classmethod
    def is_sidecar(cls, value: str = None):
        return cls.SIDECAR == (value or cls.SERVICE)

    @classmethod
    def is_streams(cls, value: str = None):
        return cls.STREAMS == (value or cls.SERVICE)

    @classmethod
    def is_api(cls, value: str = None):
        return cls.API == (value or cls.SERVICE)

    @classmethod
    def is_gateway(cls, value: str = None):
        return cls.GATEWAY == (value or cls.SERVICE)

    @classmethod
    def is_events_handlers(cls, value: str = None):
        return cls.EVENTS_HANDLER == (value or cls.SERVICE)



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


def memoize(func):
    """
    Decorator: provide result cache for wrapped func. Results are cached
    for given parameter list.

    See also: http://en.wikipedia.org/wiki/Memoization

    Example:

        @memoize
        def foo(param1, param2):
            return do_calculation(param1, param2)

        a = foo(1, 2)
        b = foo(1, 2)

        The second call is a cache hit and does not lead to a 'do_calculation'
        call.

    HINT: - The current implementation does not allow to clear the cache.
            Therefore, the decorator should only be used in limited scope,
            e.g. to enhance a locally defined methods.
          - The decorator does not work with keyword arguments.
    """

    cache = {}

    def wrapper(*x):
        if x not in cache:
            cache[x] = func(*x)
        return cache[x]

    return wrapper


def memoize_method(func):
    """
    Provides memoization for methods on a specific instance.
    Results are cached for given parameter list.

    See also: http://en.wikipedia.org/wiki/Memoization

    N.B. The cache object gets added to the instance instead of the global scope.
    Therefore cached results are restricted to that instance.
    The cache dictionary gets a name containing the name of the decorated function to
    avoid clashes.

    Example:

        class MyClass:
            @memoize
            def foo(self, a, b):
                return self._do_calculation(a, b)

    HINT: - The decorator does not work with keyword arguments.
    """

    cache_name = "__CACHED_{}".format(func.__name__)

    def wrapper(self, *args):
        cache = getattr(self, cache_name, None)
        if cache is None:
            cache = {}
            setattr(self, cache_name, cache)
        if args not in cache:
            cache[args] = func(self, *args)
        return cache[args]

    return wrapper



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import List, Optional

from polyaxon.auxiliaries import V1PolyaxonInitContainer
from polyaxon.containers.names import (
    INIT_TENSORBOARD_CONTAINER_PREFIX,
    generate_container_name,
)
from polyaxon.contexts import paths as ctx_paths
from polyaxon.k8s import k8s_schemas
from polyaxon.polypod.common import constants
from polyaxon.polypod.common.env_vars import get_run_instance_env_var
from polyaxon.polypod.common.mounts import (
    get_auth_context_mount,
    get_connections_context_mount,
)
from polyaxon.polypod.common.volumes import get_volume_name
from polyaxon.polypod.init.store import get_base_store_container
from polyaxon.polypod.specs.contexts import PluginsContextsSpec
from polyaxon.schemas.types import V1ConnectionType, V1TensorboardType
from polyaxon.utils.list_utils import to_list
from polyaxon.utils.validation import validate_tags


def _get_args(tb_args: V1TensorboardType):
    args = []
    if tb_args.port:
        args.append("--port={}".format(tb_args.port))
    if tb_args.uuids:
        uuids = validate_tags(tb_args.uuids, validate_yaml=True)
        args.append("--uuids={}".format(",".join(uuids)))
    if tb_args.use_names:
        args.append("--use-names")
    if tb_args.path_prefix:
        args.append("--path-prefix={}".format(tb_args.path_prefix)),
    if tb_args.plugins:
        plugins = validate_tags(tb_args.plugins, validate_yaml=True)
        args.append("--plugins={}".format(",".join(plugins)))

    return args


def get_tensorboard_init_container(
    polyaxon_init: V1PolyaxonInitContainer,
    artifacts_store: V1ConnectionType,
    tb_args: V1TensorboardType,
    contexts: PluginsContextsSpec,
    run_instance: str,
    container: Optional[k8s_schemas.V1Container] = None,
    env: List[k8s_schemas.V1EnvVar] = None,
    mount_path: Optional[str] = None,
) -> k8s_schemas.V1Container:
    env = to_list(env, check_none=True)
    env = env + [get_run_instance_env_var(run_instance)]

    container_name = generate_container_name(INIT_TENSORBOARD_CONTAINER_PREFIX)
    if not container:
        container = k8s_schemas.V1Container(name=container_name)

    volume_name = (
        get_volume_name(mount_path) if mount_path else constants.VOLUME_MOUNT_ARTIFACTS
    )
    mount_path = mount_path or ctx_paths.CONTEXT_MOUNT_ARTIFACTS
    volume_mounts = [
        get_connections_context_mount(name=volume_name, mount_path=mount_path)
    ]
    if contexts and contexts.auth:
        volume_mounts.append(get_auth_context_mount(read_only=True))

    args = [
        "--context-from={}".format(artifacts_store.store_path),
        "--context-to={}".format(mount_path),
        "--connection-kind={}".format(artifacts_store.kind),
    ]
    args += _get_args(tb_args)

    return get_base_store_container(
        container=container,
        container_name=container_name,
        polyaxon_init=polyaxon_init,
        store=artifacts_store,
        command=["polyaxon", "initializer", "tensorboard"],
        args=args,
        env=env,
        env_from=[],
        volume_mounts=volume_mounts,
    )



#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from polyaxon.connections.schemas.connections import (
    BucketConnectionSchema,
    ClaimConnectionSchema,
    ConnectionSchema,
    GitConnectionSchema,
    HostConnectionSchema,
    HostPathConnectionSchema,
    V1BucketConnection,
    V1ClaimConnection,
    V1GitConnection,
    V1HostConnection,
    V1HostPathConnection,
    validate_connection,
)
from polyaxon.connections.schemas.k8s_resources import (
    K8sResourceSchema,
    V1K8sResourceSchema,
    validate_k8s_resource,
)
